/*
 * Copyright (C), 2017-2022, tuya. All rights reserved.
 * File name  : ty_list.c
 * Author     : clb
 * Date       : 2017/05/31
 * Description: ty_list
 * Others     :
 *
 * $ID: ty_list.c, v1.0.0 2017/05/31 clb create $
 */
#include <stdio.h>
#include <stdlib.h>
#include "ty_list.h"

#define HEAD node.next     /* first node in list */
#define TAIL node.previous /* last node in list */

/** @fn: lstInit
 *   @brief: 链表初始化
 *   @param [in]pList
 *   @return
 */
void lstInit(LIST *pList)
{
    pList->HEAD = NULL;
    pList->TAIL = NULL;
    pList->count = 0;
}

/** @fn: lstAdd
 *   @brief: 链表添加节点,添加到链表尾
 *   @param [in]pList
 *   @return
 */
void lstAdd(LIST *pList, NODE *pNode)
{
    lstInsert(pList, pList->TAIL, pNode);
}

void lstConcat(LIST *pDstList, LIST *pAddList)
{
    if (pAddList->count == 0) /* nothing to do if AddList is empty */
        return;

    if (pDstList->count == 0)
        *pDstList = *pAddList;
    else {
        /* both lists non-empty; update DstList pointers */

        pDstList->TAIL->next = pAddList->HEAD;
        pAddList->HEAD->previous = pDstList->TAIL;
        pDstList->TAIL = pAddList->TAIL;

        pDstList->count += pAddList->count;
    }

    /* make AddList empty */

    lstInit(pAddList);
}

/** @fn: lstCount
 *   @brief: 链表节点数目
 *   @param [in]pList
 *   @return count
 */
int lstCount(LIST *pList)
{
    return (pList->count);
}

/** @fn: lstDelete
 *   @brief: 链表删除节点
 *   @param [in]pList pNode
 *   @return
 */
void lstDelete(LIST *pList, NODE *pNode)
{

    int firstNode = 0, lastNode = 0;

    if (pNode->previous == NULL) {
        if (pList->HEAD != pNode) {
            printf("lstDelete: HEAD = %p, pNode = %p, it's a dummy node!\n", pList->HEAD, pNode);
            return;
        }

        firstNode = 1;
    } else {
        if (pNode->previous->next != pNode) {
            printf("lstDelete: previous->next = %p, pNode = %p, it's a dummy node!\n", pNode->previous->next, pNode);
            return;
        }

        firstNode = 0;
    }

    if (pNode->next == NULL) {
        if (pList->TAIL != pNode) {
            printf("lstDelete: TAIL = %p, pNode = %p, it's a dummy node!\n", pList->TAIL, pNode);
            return;
        }

        lastNode = 1;
    } else {
        if (pNode->next->previous != pNode) {
            printf("lstDelete: next->previous = %p, pNode = %p, it's a dummy node!\n", pNode->next->previous, pNode);
            return;
        }

        lastNode = 0;
    }

    if (1 == firstNode) {
        pList->HEAD = pNode->next;
    } else {
        pNode->previous->next = pNode->next;
    }

    if (1 == lastNode) {
        pList->TAIL = pNode->previous;
    } else {
        pNode->next->previous = pNode->previous;
    }

    /* update node count */

    pList->count--;
}

/** @fn: lstFirst
 *   @brief: 取链表头数据
 *   @param [in]pList pNode
 *   @return
 */
NODE *lstFirst(LIST *pList)
{
    return (pList->HEAD);
}

/** @fn: lstGet
 *   @brief: 取链表头节点，取出后会删除对应节点
 *   @param [in]pList pNode
 *   @return
 */
NODE *lstGet(LIST *pList)
{
    NODE *pNode = pList->HEAD;

    if (pNode != NULL) /* is list empty? */
    {
        pList->HEAD = pNode->next; /* make next node be 1st */

        if (pNode->next == NULL) /* is there any next node? */
            pList->TAIL = NULL;  /*   no - list is empty */
        else
            pNode->next->previous = NULL; /*   yes - make it 1st node */

        pList->count--; /* update node count */
    }

    return (pNode);
}

/** @fn: lstInsert
 *   @brief: 链表节点插入
 *   @param [in]pList pNode
 *   @return
 */
void lstInsert(LIST *pList, NODE *pPrev, NODE *pNode)
{
    NODE *pNext;

    if (pPrev == NULL) { /* new node is to be first in list */
        pNext = pList->HEAD;
        pList->HEAD = pNode;
    } else { /* make prev node point fwd to new */
        pNext = pPrev->next;
        pPrev->next = pNode;
    }

    if (pNext == NULL)
        pList->TAIL = pNode; /* new node is to be last in list */
    else
        pNext->previous = pNode; /* make next node point back to new */

    /* set pointers in new node, and update node count */

    pNode->next = pNext;
    pNode->previous = pPrev;

    pList->count++;
}

/** @fn: lstLast
 *   @brief: 取链表最后节点
 *   @param [in]pList
 *   @return
 */
NODE *lstLast(LIST *pList)
{
    return (pList->TAIL);
}

/** @fn: lstNext
 *   @brief: 节点下一个节点
 *   @param [in]pNode
 *   @return
 */
NODE *lstNext(NODE *pNode)
{
    return (pNode->next);
}

/** @fn: lstNth
 *   @brief: 取链表第N个节点
 *   @param [in]pList
 *   @return
 */
NODE *lstNth(LIST *pList, int nodenum)
{
    NODE *pNode;

    /* verify node number is in list */

    if ((nodenum < 1) || (nodenum > pList->count))
        return (NULL);

    /* if nodenum is less than half way, look forward from beginning;
       otherwise look back from end */

    if (nodenum < (pList->count >> 1)) {
        pNode = pList->HEAD;

        while (--nodenum > 0)
            pNode = pNode->next;
    }

    else {
        nodenum -= pList->count;
        pNode = pList->TAIL;

        while (nodenum++ < 0)
            pNode = pNode->previous;
    }

    return (pNode);
}

/** @fn: lstPrevious
 *   @brief: 取节点的前置节点
 *   @param [in]pNode
 *   @return
 */
NODE *lstPrevious(NODE *pNode)
{
    return (pNode->previous);
}

/** @fn: lstNStep
 *   @brief: 取节点的前置(后置)节点
 *   @param [in]pNode
 *   @return
 */
NODE *lstNStep(NODE *pNode, int nStep)
{
    int i;

    for (i = 0; i < abs(nStep); i++) {
        if (nStep < 0)
            pNode = pNode->previous;
        else if (nStep > 0)
            pNode = pNode->next;
        if (pNode == NULL)
            break;
    }
    return (pNode);
}

/** @fn: lstFind
 *   @brief: 链表中查询节点
 *   @param [in]pList
 *   @return
 */
int lstFind(LIST *pList, NODE *pNode)
{

    NODE *pNextNode;
    int index = 1;

    pNextNode = lstFirst(pList);

    while ((pNextNode != NULL) && (pNextNode != pNode)) {
        index++;
        pNextNode = lstNext(pNextNode);
    }

    if (pNextNode == NULL)
        return (-1);
    else
        return (index);
}

/** @fn: lstFree
 *   @brief: 链表中释放
 *   @param [in]pList
 *   @return
 */

void lstFree(LIST *pList)
{
    NODE *p1, *p2;

    if (pList->count > 0) {
        p1 = pList->HEAD;
        while (p1 != NULL) {
            p2 = p1->next;
            free((char *)p1);
            p1 = p2;
        }
        pList->count = 0;
        pList->HEAD = pList->TAIL = NULL;
    }
}
