/**
 * @file tal_audio.h
 * @brief Common process - the audio api
 * @version 0.1
 * @date 2021-11-09
 *
 * @copyright Copyright 2019-2021 Tuya Inc. All Rights Reserved.
 *
 */

#include <stdio.h>
#include "tal_audio.h"
#include "tuya_iot_config.h"
#include "tuya_error_code.h"
#include "uni_log.h"

/**
 * @brief ai init
 *
 * @param[in] pconfig: audio config
 * @param[in] count: count of pconfig
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ai_init(TAL_AUDIO_CONFIG_T *pconfig, INT32_T count)
{
    INT32_T ret = 0;
    if (pconfig == NULL || count < 1) {
        PR_ERR("int parg is wrong, pconfig:%p count:%d", pconfig, count);
        return OPRT_COM_ERROR;
    }
    ret = tkl_ai_init(pconfig, count);
    if (ret != 0) {
        PR_ERR("ai init is failed, ret = %d\n", ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ai start
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ai_start(INT32_T card, INT32_T chn)
{
    INT32_T ret = -1;
    ret = tkl_ai_start(card, (TKL_AI_CHN_E)chn);
    if (ret != 0) {
        PR_ERR("tkl_ai_start %d %d failed, ret = %d\n", card, chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ai set
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[in] cmd
 * @param[in] parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ai_set(INT32_T card, INT32_T chn, TAL_AI_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    INT32_T vol = 0;

    if (NULL == parg) {
        PR_ERR("parg is null, cmd:%d\n", cmd);
        return OPRT_COM_ERROR;
    }
    switch (cmd) {
    case TAL_AI_CMD_VOL:
        vol = *(int *)parg;
        ret = tkl_ai_set_vol(card, (TKL_AI_CHN_E)chn, vol);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vol %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_AEC:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_AEC, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_NR:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_NR, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_HPF:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_HPF, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_AGC:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_AGC, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_ALC:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_ALC, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_AI_CMD_VQE_HS:
        ret = tkl_ai_set_vqe(card, (TKL_AI_CHN_E)chn, TKL_AUDIO_VQE_HS, (TKL_AUDIO_VQE_PARAM_T*)parg);
        if (ret != 0) {
            PR_ERR("tkl_ai_set_vqe %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }

    return OPRT_OK;
}

/**
 * @brief ai get frame
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[out] pframe: audio frame, pframe->pbuf allocated by upper layer application
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ai_get_frame(INT32_T card, INT32_T chn, TAL_AUDIO_FRAME_INFO_T *pframe)
{
    INT32_T ret = -1;
    ret = tkl_ai_get_frame(card, (TKL_AI_CHN_E)chn, pframe);
    if (ret != 0) {
        // PR_ERR("tkl_ai_get_frame %d %d failed, ret = %d\n", card, chn);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/***********************************************************
 *  函数名: tal_ai_stop
 *  功能：停止音频输入
 *  参数: Input，card声卡号，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tal_ai_stop(INT32_T card, INT32_T chn)
{
    INT32_T ret = -1;
    ret = tkl_ai_stop(card, (TKL_AI_CHN_E)chn);
    if (ret != 0) {
        PR_ERR("tkl_ai_stop %d %d failed, ret = %d\n", card, chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ai uninit
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ai_uninit(VOID)
{
    INT32_T ret = -1;
    ret = tkl_ai_uninit();
    if (ret != 0) {
        PR_ERR("tkl_ai_uninit failed \n");
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ao init
 *
 * @param[in] pconfig: audio config
 * @param[in] count: config count
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_init(TAL_AUDIO_CONFIG_T *pconfig, INT32_T count, VOID **handle)
{
    INT32_T ret = 0;
    if (pconfig == NULL || count < 1) {
        PR_ERR("int parg is wrong, pconfig:%p count:%d", pconfig, count);
        return OPRT_COM_ERROR;
    }

    ret = tkl_ao_init(pconfig, count, handle);
    if (ret != 0) {
        PR_ERR("tkl_ao_init failed, ret = %d\n", ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ao start
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[out] handle: handle of start
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_start(INT32_T card, INT32_T chn, VOID *handle)
{
    INT32_T ret = -1;
    ret = tkl_ao_start(card, (TKL_AO_CHN_E)chn, handle);
    if (ret != 0) {
        PR_ERR("tkl_ao_start %d %d failed, ret = %d\n", card, chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ao set
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[in] handle: the return of start
 * @param[in] cmd
 * @param[in] parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_set(INT32_T card, INT32_T chn, void *handle, TAL_AO_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    INT32_T vol = 0;
    switch (cmd) {
    case TAL_AO_CMD_VOL:
        vol = *(int *)parg;
        ret = tkl_ao_set_vol(card, (TKL_AO_CHN_E)chn, handle, vol);
        if (ret != 0) {
            PR_ERR("tkl_ao_set_vol %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }

    return OPRT_OK;
}

/**
 * @brief ao get
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[in] handle: the return of start
 * @param[in] cmd
 * @param[in] parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_get(INT32_T card, INT32_T chn, void *handle, TAL_AO_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    INT32_T vol = 0;
    switch (cmd) {
    case TAL_AO_CMD_VOL:
        ret = tkl_ao_get_vol(card, (TKL_AO_CHN_E)chn, handle, &vol);
        if (ret != 0) {
            PR_ERR("tkl_ao_set_vol %d %d failed, ret = %d\n", card, chn, ret);
            return OPRT_COM_ERROR;
        }
        *(INT32_T *)parg = vol;
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }

    return OPRT_OK;
}

/**
 * @brief ao output frame
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[in] handle: handle of start
 * @param[in] pframe: output frame
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_put_frame(INT32_T card, INT32_T chn, VOID *handle, TAL_AUDIO_FRAME_INFO_T *pframe)
{
    INT32_T ret = -1;
    ret = tkl_ao_put_frame(card, (TKL_AO_CHN_E)chn, handle, pframe);
    if (ret != 0) {
        PR_ERR("tkl_ao_put_frame %d %d failed, ret = %d\n", card, chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ao stop
 *
 * @param[in] card: card number
 * @param[in] chn: channel number
 * @param[in] handle: handle of start
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_stop(INT32_T card, INT32_T chn, VOID *handle)
{
    INT32_T ret = -1;
    ret = tkl_ao_stop(card, (TKL_AO_CHN_E)chn, handle);
    if (ret != 0) {
        PR_ERR("tkl_ao_stop %d %d failed, ret = %d\n", card, chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief ao uninit
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_ao_uninit(VOID *handle)
{
    INT32_T ret = -1;
    ret = tkl_ao_uninit(handle);
    if (ret != 0) {
        PR_ERR("tkl_ao_uninit failed \n");
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

