/**
 * @file tal_video_enc.h
 * @brief video encode
 * @version 0.1
 * @date 2021-11-09
 *
 * @copyright Copyright 2019-2021 Tuya Inc. All Rights Reserved.
 *
 * Set video parameters and get video bitstream.
 */

#include <stdio.h>
#include "tal_video_enc.h"
#include "tuya_iot_config.h"
#include "tuya_error_code.h"
#include "uni_log.h"

/**
 * @brief video encode init
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] pconfig: venc config
 * @param[in] count: count of pconfig
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_init(INT32_T vi_chn, TAL_VENC_CONFIG_T *pconfig, INT32_T count)
{
    INT32_T ret = 0;
    ret = tkl_venc_init(vi_chn, pconfig, count);
    if (ret != 0) {
        PR_ERR("tkl_venc_init failed, ret = %d\n", ret);
        return OPRT_COM_ERROR;
    }

    return OPRT_OK;
}

/**
 * @brief video set
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc chn
 * @param[in] cmd: cmd
 * @param[in] parg: parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_set(INT32_T vi_chn, INT32_T venc_chn, TAL_VENC_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    if ((NULL == parg) 
        && (cmd != TAL_VENC_CMD_START)
        && (cmd != TAL_VENC_CMD_STOP)) {
        PR_ERR("parg is null, cmd:%d\n", cmd);
        return OPRT_COM_ERROR;
    }
    switch (cmd) {
    case TAL_VENC_CMD_OSD:
        ret = tkl_venc_set_osd((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, (TKL_VENC_OSD_T *)parg);
        if (0 != ret) {
            PR_ERR("ty_dev_venc_osd failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_IDR:
        ret = tkl_venc_set_IDR((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, *(TKL_VENC_IDR_E *)parg);
        if (0 != ret) {
            PR_ERR("tkl_venc_set_IDR failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_STREAM_BUFF:
        ret = tkl_venc_set_video_stream_buffer((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn,
                                               (TAL_VENC_STREAM_BUFF_T *)parg);
        if (0 != ret) {
            PR_ERR("tkl_venc_set_video_stream_buffer failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_START:
        ret = tkl_venc_start((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn);
        if (0 != ret) {
            PR_ERR("tkl_venc_start failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_STOP:
        ret = tkl_venc_stop((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn);
        if (0 != ret) {
            PR_ERR("tkl_venc_stop failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_MASK:
        ret = tkl_venc_set_mask((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, (TAL_VENC_MASK_T *)parg);
        if (0 != ret) {
            PR_ERR("tkl_venc_set_mask failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    case TAL_VENC_CMD_FORMAT:
        ret = tkl_venc_set_format((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, (TAL_VENC_FORMAT_T *)parg);
        if (0 != ret) {
            PR_ERR("tkl_venc_set_format failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }
    return OPRT_OK;
}

/**
 * @brief video get
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc chn
 * @param[in] cmd: cmd
 * @param[out] parg: parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_get(INT32_T vi_chn, INT32_T venc_chn, TAL_VENC_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    if (NULL == parg) {
        PR_ERR("parg is null, cmd:%d\n", cmd);
        return OPRT_COM_ERROR;
    }

    switch (cmd) {
    case TAL_VENC_CMD_FORMAT:
        ret = tkl_venc_get_format((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, (TAL_VENC_FORMAT_T *)parg);
        if (0 != ret) {
            PR_ERR("tkl_venc_get_format failed,%d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }

    return OPRT_OK;
}

/**
 * @brief video encode get frame
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc channel number
 * @param[out] pframe: output frame
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_get_frame(INT32_T vi_chn, INT32_T venc_chn, TAL_VENC_FRAME_T *pframe)
{
    INT32_T ret = -1;
    ret = tkl_venc_get_frame((TKL_VI_CHN_E)vi_chn, (TKL_VENC_CHN_E)venc_chn, pframe);
    if (ret != 0) {
        // PR_ERR("tkl_venc_get_frame %d %d failed, ret = %d\n", vi_chn, venc_chn,ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief video first snap
 *
 * @param[in] vi_chn: vi channel number
 * @param[out] pframe: output frame
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_get_first_snap(INT32_T vi_chn, TKL_VENC_FRAME_T *pframe)
{
    INT32_T ret = -1;
    ret = tkl_venc_get_first_snap((TKL_VI_CHN_E)vi_chn, pframe);
    if (ret != 0) {
        PR_ERR("tkl_venc_get_first_snap %d failed, ret = %d\n", vi_chn, ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief set video time callback
 *        Used to set osd time
 * @param[in] cb: time callback api
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_set_time_cb(TKL_VENC_TIME_CB cb)
{
    int ret = tkl_venc_set_time_cb(cb);
    if (ret != 0) {
        PR_ERR("tkl_venc_set_time_cb failed, ret = %d\n", ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/**
 * @brief video encode uninit
 *
 * @param[in] vi_chn: vi channel number
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_venc_uninit(INT32_T vi_chn)
{
    INT32_T ret = -1;
    ret = tkl_venc_uninit((TKL_VI_CHN_E)vi_chn);
    if (ret != 0) {
        PR_ERR("tkl_venc_uninit failed \n");
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}
