/*
 * Copyright (C), 2020-2022, tuya. All rights reserved.
 * File name  : KalmanTracker.c
 * Author     : changde
 * Date       : 2022/11/22
 * Description: 
 * Others     :
 */
#include <sys/time.h>
#include <stdio.h>
#include "KalmanTracker.h"
#include "mat.h"
#include <float.h>
#include <stdlib.h>
#include <string.h>

static int kf_count;

typedef struct {
	MatPtr transitionMatrix, measurementMatrix, processNoiseCov, measurementNoiseCov, errorCovPost, statePost;
	MatPtr state_pre, error_cov_pre;
	KALMAN_FILETER_STATE state;
	KALMAN_PRIV priv;
} KALMAN_FILETER_HDL;


float mysqrt_3(float x)
{
    float xhalf = 0.5f*x;
    int i = *(int*)&x; 
    
    if(x < FLT_EPSILON) {
		return 0;
	}

    i = 0x5f375a86 - (i>>1); // beautiful number
    x = *(float*)&i; 
    x = x*(1.5f-xhalf*x*x); // 牛顿迭代法，提高精度
    x = x*(1.5f-xhalf*x*x); // 牛顿迭代法，提高精度
    x = x*(1.5f-xhalf*x*x); // 牛顿迭代法，提高精度

    return 1/x;
}

// Convert bounding box from [cx,cy,s,r] to [x,y,w,h] style.
static void _get_rect_xysr(KALMAN_RECT *rect, float cx, float cy, float s, float r)
{
	float w = mysqrt_3(s * r);
	float h = s / w;
	float x = (cx - w / 2);
	float y = (cy - h / 2);

	if (x < 0 && cx > 0)
		x = 0;
	if (y < 0 && cy > 0)
		y = 0;

	rect->x = x;
	rect->y = y;
	rect->width = w;
	rect->height = h;

	return;
}

// initialize Kalman filter
static void _init_kf(KALMAN_FILETER_HDL *hdl, KALMAN_RECT *initRect)
{
	int stateNum = 7;
	int measureNum = 4;

	hdl->state_pre = CreateMatrix(stateNum, 1);
	hdl->statePost = CreateMatrix(stateNum, 1);
	hdl->error_cov_pre = CreateMatrix(stateNum, stateNum);
	hdl->transitionMatrix = CreateMatrix(stateNum, stateNum);
    hdl->measurementMatrix = CreateMatrix(measureNum, stateNum);
    hdl->processNoiseCov = CreateMatrix(stateNum, stateNum);
   	hdl-> measurementNoiseCov = CreateMatrix(measureNum, measureNum);
    hdl->errorCovPost = CreateMatrix(stateNum, stateNum);
	hdl->transitionMatrix->mat[0][4] = 1;
	hdl->transitionMatrix->mat[1][5] = 1;
	hdl->transitionMatrix->mat[2][6] = 1;

	SetIdentityMatrix(hdl->transitionMatrix, 1);       //A
	SetIdentityMatrix(hdl->measurementMatrix, 1);      //H
	SetIdentityMatrix(hdl->processNoiseCov, 0.01);     //Q
	SetIdentityMatrix(hdl->measurementNoiseCov, 0.1);  //R
	SetIdentityMatrix(hdl->errorCovPost, 1);           //R

	hdl->statePost->mat[0][0] = initRect->x + initRect->width / 2;
	hdl->statePost->mat[1][0] = initRect->y + initRect->height / 2;
	hdl->statePost->mat[2][0] = initRect->width * initRect->height;
	hdl->statePost->mat[3][0] = initRect->width / initRect->height;

	return;
}

void kalman_tracker_reset()
{
	kf_count = 0;
	return;
}

void *kalman_tracker_create(KALMAN_RECT *initRect, KALMAN_PRIV *p_priv)
{
	if(initRect == NULL) {
		printf("ERR initRect NULL\n");
		return NULL;
	}

	KALMAN_FILETER_HDL *hdl = (KALMAN_FILETER_HDL *)malloc(sizeof(KALMAN_FILETER_HDL));
	memset(hdl, 0, sizeof(KALMAN_FILETER_HDL));
	if (hdl == NULL || (p_priv == NULL)) {
		printf("ERR kalman_tracker_create malloc failed\n");
		return NULL;
	}

	hdl->priv = *p_priv;;
	KALMAN_FILETER_STATE *pstat = &hdl->state;

	pstat->m_time_since_update = 0;
	pstat->m_hits = 0;
	pstat->m_hit_streak = 0;
	pstat->m_age = 0;
	pstat->m_id = kf_count;
	pstat->lastRect = *initRect;
	kf_count++;

	_init_kf(hdl, initRect);
	return hdl;
}

void kalman_tracker_destory(void *hdl)
{
	KALMAN_FILETER_HDL *k_hdl = (KALMAN_FILETER_HDL *)hdl;

	if(hdl == NULL) {
		printf("kalman_tracker_destory err null input\n");
		return;
	}

	DestroyMatrix(k_hdl->state_pre);
	DestroyMatrix(k_hdl->statePost);
	DestroyMatrix(k_hdl->error_cov_pre);
	DestroyMatrix(k_hdl->transitionMatrix);
	DestroyMatrix(k_hdl->measurementMatrix);
	DestroyMatrix(k_hdl->processNoiseCov);
	DestroyMatrix(k_hdl->measurementNoiseCov);
	DestroyMatrix(k_hdl->errorCovPost);

	free(hdl);

	return;
}

// Predict the estimated bounding box.
void kalman_tracker_predict(void *hdl, KALMAN_RECT *result)
{
	KALMAN_FILETER_HDL *k_hdl = (KALMAN_FILETER_HDL *)hdl;
	if ((hdl == NULL) || (result == NULL)) {
		printf("kalman_tracker_predict NULL input\n");
		return;
	}

	/* update the state */
    /* x'(k) = A*x(k) */
	DestroyMatrix(k_hdl->state_pre);
	k_hdl->state_pre = ProductMatrix(k_hdl->transitionMatrix, k_hdl->statePost);
    /* update error covariance matrices */
    /* temp1 = A*P(k) */
	MatPtr temp1 = ProductMatrix(k_hdl->transitionMatrix, k_hdl->errorCovPost);    /* P'(k) = temp1*At + Q */
	MatPtr At = TransMatrix(k_hdl->transitionMatrix);
	MatPtr temp2 = ProductMatrix(temp1, At);
	DestroyMatrix(At);
	DestroyMatrix(temp1);
	DestroyMatrix(k_hdl->error_cov_pre);
	k_hdl->error_cov_pre = AddMatrix(temp2, k_hdl->processNoiseCov);
	DestroyMatrix(temp2);

	KALMAN_RECT rect = {0};
	_get_rect_xysr(&rect, k_hdl->state_pre->mat[0][0], k_hdl->state_pre->mat[1][0], k_hdl->state_pre->mat[2][0], k_hdl->state_pre->mat[3][0]);
	//k_hdl->m_history.push_back(rect);

	*result = rect;

	KALMAN_FILETER_STATE *pstat = &k_hdl->state;

	pstat->m_age += 1;
	if (pstat->m_time_since_update > 0) {
		pstat->m_hit_streak = 0;
	}
	pstat->m_time_since_update += 1;

	return;
}

// Update the state vector with observed bounding box.
void kalman_tracker_update(void *hdl, KALMAN_RECT *stateMat, KALMAN_PRIV *p_priv)
{	
	KALMAN_FILETER_HDL *k_hdl = (KALMAN_FILETER_HDL *)hdl;
	if ((hdl == NULL) || (stateMat == NULL) || (p_priv == NULL)) {
		printf("kalman_tracker_update NULL input\n");
		return;
	}
	//k_hdl->m_history.clear();
	k_hdl->priv = *p_priv;

	KALMAN_FILETER_STATE *pstat = &k_hdl->state;
	pstat->m_time_since_update = 0;
	pstat->m_hits += 1;
	pstat->m_hit_streak += 1;
	pstat->lastRect = *stateMat;

	// measurement
	static MatPtr measurement = NULL;
	if (measurement == NULL) {
		measurement = CreateMatrix(4, 1);
		if (measurement == NULL) { 
			printf("measurement CreateMatrix failed\n");
			return;
		}
	} 
	measurement->mat[0][0] = stateMat->x + stateMat->width / 2;
	measurement->mat[1][0] = stateMat->y + stateMat->height / 2;
	measurement->mat[2][0] = stateMat->width * stateMat->height;
	measurement->mat[3][0] = stateMat->width / stateMat->height;

    /* temp2 = H*P'(k) */
	MatPtr temp2 = ProductMatrix(k_hdl->measurementMatrix, k_hdl->error_cov_pre);

    /* temp3 = temp2*Ht + R */
	MatPtr Ht = TransMatrix(k_hdl->measurementMatrix);
	MatPtr mid = ProductMatrix(temp2, Ht);
	MatPtr temp3 = AddMatrix(mid, k_hdl->measurementNoiseCov);

    // temp4 = inv(temp3)*temp2 = Kt(k)
	MatPtr inv_temp3 = InvertMatrix(temp3);
	MatPtr temp4 = ProductMatrix(inv_temp3, temp2);

    // K(k)
    MatPtr gain = TransMatrix(temp4);

    // temp5 = z(k) - H*x'(k)
	MatPtr mid1 = ProductMatrix(k_hdl->measurementMatrix, k_hdl->state_pre);
    MatPtr temp5 = SubMatrix(measurement, mid1);

    // x(k) = x'(k) + K(k)*temp5
	MatPtr mid2 = ProductMatrix(gain, temp5);
	DestroyMatrix(k_hdl->statePost);
    k_hdl->statePost = AddMatrix(k_hdl->state_pre, mid2);

    // P(k) = P'(k) - K(k)*temp2
	MatPtr mid3 = ProductMatrix(gain, temp2);
	DestroyMatrix(k_hdl->errorCovPost);
	k_hdl->errorCovPost = SubMatrix(k_hdl->error_cov_pre, mid3);

	DestroyMatrix(Ht);
	DestroyMatrix(inv_temp3);
	DestroyMatrix(gain);
	DestroyMatrix(mid);
	DestroyMatrix(mid1);
	DestroyMatrix(mid2);
	DestroyMatrix(mid3);
	DestroyMatrix(temp2);
	DestroyMatrix(temp3);
	DestroyMatrix(temp4);
	DestroyMatrix(temp5);

    //result = kalman->statePost;
	
	return;
}

void kalman_tracker_state(void *hdl, KALMAN_FILETER_STATE *stat, KALMAN_PRIV *p_priv)
{
	KALMAN_FILETER_HDL *k_hdl = (KALMAN_FILETER_HDL *)hdl;
	if ((hdl == NULL) || (stat == NULL) || (p_priv == NULL)) {
		printf("kalman_tracker_state NULL input\n");
		return;
	}

	*stat = k_hdl->state;
	*p_priv = k_hdl->priv;
	return;
}
