/*
 * Copyright (C), 2020-2022, tuya. All rights reserved.
 * File name  : ty_target_track.c
 * Author     : changde
 * Date       : 2024/2/5
 * Description: 目标追踪，基于SORT算法
 * Others     :
 */
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "ty_target_track.h"
#include "Hungarian.h"
#include "float.h"

typedef struct{
    int max_age;
    int min_hits;
    float iou_thrs;
    int frame_count;
    int tracker_num;
    void * trackers[MAX_TRACKERS_NUM];
    SORT_BOX predicted_boxes[MAX_TRACKERS_NUM];
    int assignment[MAX_TRACKERS_NUM];
    int unmatch_dets[MAX_TRACKERS_NUM];
    float *iou_matrix;
} SORT_HDL;

static void _erase_tracker(SORT_HDL *hdl, int idx)
{
    int i = 0;
    kalman_tracker_destory(hdl->trackers[idx]);
    for (i = idx; i < hdl->tracker_num - 1; i++) {
        hdl->trackers[i] = hdl->trackers[i+1];
    }
    hdl->tracker_num--;
    if (hdl->tracker_num < 0 ) {
        hdl->tracker_num = 0; 
    }

    return;
}

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))

static float _get_IOU(SORT_BOX *bb_test, SORT_BOX *bb_gt) {
    float width = MIN(bb_test->x, bb_gt->x) + bb_test->width + bb_gt->width - MAX(bb_test->x + bb_test->width, bb_gt->x + bb_gt->width);
    float height = MIN(bb_test->y, bb_gt->y) + bb_test->height + bb_gt->height - MAX(bb_test->y + bb_test->height, bb_gt->y + bb_gt->height);
    float in =  width * height;
    float un = bb_test->width * bb_test->height + bb_gt->width * bb_gt->height - in;

    if ((width <= 0) || (height <= 0)) {
        return 0;
    }

    if(in > un) {
        return 1;
    }

    if (un < FLT_EPSILON) {
        return 0;
    }

    return (in / un);
}

int sort_tracker_update(void *sort_hdl, SORT_TRACKING_LIST *in, SORT_TRACKING_LIST *out)
{
    int i = 0;
    int j = 0;
    int num = 0;
    SORT_HDL *hdl = (SORT_HDL *)sort_hdl;

    if ((in == NULL) || (out == NULL) || (hdl == NULL)) {
        printf("err sort_tracker_update input NULL\n");
        return -1;
    }

    if(in->num > MAX_TRACKERS_NUM) {
        printf("err sort_tracker_update num > MAX_TRACKERS_NUM %d\n", MAX_TRACKERS_NUM);
        return -1;
    }

    hdl->frame_count++;

    num = in->num;
    memset(out, 0, sizeof(SORT_TRACKING_LIST));

    if (hdl->tracker_num == 0) { // the first frame met
        // initialize kalman trackers using first detections.
        for (i = 0; i < num; i++) {
            KALMAN_RECT rect = {0};
            rect.x = in->item[i].box.x;
            rect.y = in->item[i].box.y;
            rect.width = in->item[i].box.width;
            rect.height = in->item[i].box.height;
            void *trk = kalman_tracker_create(&rect, &in->item[i].priv);
            hdl->trackers[hdl->tracker_num++] = trk;
            out->item[i].id = 0;
            if (hdl->tracker_num == MAX_TRACKERS_NUM) {
                break;
            }
        }
        return 0;
    }

    // 3.1. get predicted locations from existing trackers. 从当前跟踪器，获取预测框
    int trkNum = 0;
    for (i = 0; i < hdl->tracker_num;) {
        KALMAN_RECT rect = {0};
        kalman_tracker_predict(hdl->trackers[i], &rect);   // 根据上次的结果，预测当前的位置
        if (rect.x >= 0 && rect.y >= 0 && rect.width != 0) {
            SORT_BOX tmp_box;
            tmp_box.x = rect.x;
            tmp_box.y = rect.y;
            tmp_box.width = rect.width;
            tmp_box.height = rect.height;

            hdl->predicted_boxes[trkNum++] = tmp_box; //trkNum不会超过MAX
            i++;
        } else {
            _erase_tracker(hdl, i);
        }
    }

    // 3.2. associate detections to tracked object (both represented as bounding boxes) 检测目标和跟踪目标关联
    int detNum = num;
    float *iou_matrix = hdl->iou_matrix;

    for (i = 0; i < trkNum; i++) {      // compute iou matrix as a distance matrix 
        for (j = 0; j < detNum; j++) {
            // use 1-iou because the hungarian algorithm computes a minimum-cost assignment.
            iou_matrix[i * detNum + j] = 1 - _get_IOU(&hdl->predicted_boxes[i], &in->item[j].box);
        }
    }

    // solve the assignment problem using hungarian algorithm.
    // the resulting assignment is [track(prediction) : detection], with len=preNum
    int *assignment = hdl->assignment;
    HungarianSolve(iou_matrix, trkNum, detNum, assignment);   

    // find matches, unmatched_detections and unmatched_predictions
    int *unmatch_dets= hdl->unmatch_dets;
    int unmatch_num = 0;

    uint64_t flag = 0;
    if (detNum > trkNum) {    // there are unmatched detections
        for (i = 0; i < trkNum; ++i) {
            if(assignment[i] != -1) {
                flag |= (1 << assignment[i]);
            }
        } 

        for(i = 0; i < detNum; i++) {
            if (!((flag >> i) & 0x1)) {
                unmatch_dets[unmatch_num++] = i;
            }
        }
    } 

    // filter out matched with low IOU 过滤低交占比的目标
    for (i = 0; i < trkNum; ++i) {
        if (assignment[i] == -1) { // pass over invalid values
            continue;
        }
        if (1 - iou_matrix[i * detNum + assignment[i]] < hdl->iou_thrs) {
            unmatch_dets[unmatch_num++] = assignment[i];
        } else {
            // 3.3. updating trackers
            // update matched trackers with assigned detections.
            // each prediction is corresponding to a tracker
            int trkIdx = i;
            int detIdx = assignment[i];     //// 原版detIdx:1 trkIdx:0   原版的tracker有两个
            KALMAN_RECT rect = {0};
            rect.x = in->item[detIdx].box.x;
            rect.y = in->item[detIdx].box.y;
            rect.width = in->item[detIdx].box.width;
            rect.height = in->item[detIdx].box.height;

            kalman_tracker_update(hdl->trackers[trkIdx], &rect, &in->item[detIdx].priv);
        }
    }

    // create and initialise new trackers for unmatched detections 为不匹配的检测创建并初始化新的跟踪器
    for (i = 0; i < unmatch_num; i++)
    {
        if (hdl->tracker_num == MAX_TRACKERS_NUM) {
            break;
        }

        KALMAN_RECT rect = {0};
        rect.x = in->item[unmatch_dets[i]].box.x;
        rect.y = in->item[unmatch_dets[i]].box.y;
        rect.width = in->item[unmatch_dets[i]].box.width;
        rect.height = in->item[unmatch_dets[i]].box.height;
        void *trk = kalman_tracker_create(&rect, &in->item[unmatch_dets[i]].priv);

        hdl->trackers[hdl->tracker_num++] = trk;
    }

    // get trackers' output
    int tmp_result_num = 0;
    for (i = 0; i < hdl->tracker_num; i++)
    {
        KALMAN_FILETER_STATE stat = {0};
        KALMAN_PRIV priv = {0};
        // remove dead tracklet
        kalman_tracker_state(hdl->trackers[i], &stat, &priv);
        if (stat.m_time_since_update > hdl->max_age) {
            _erase_tracker(hdl, i);
        } else {
            out->item[tmp_result_num].id = stat.m_id + 1;
            out->item[tmp_result_num].box.x = stat.lastRect.x;
            out->item[tmp_result_num].box.y = stat.lastRect.y;
            out->item[tmp_result_num].box.width = stat.lastRect.width;
            out->item[tmp_result_num].box.height = stat.lastRect.height;
            out->item[tmp_result_num].id = stat.m_id + 1;
            out->item[tmp_result_num].frame = hdl->frame_count;
            out->item[tmp_result_num].priv = priv;

            if ((stat.m_time_since_update == 0) && //当前未匹配的会输出
                (stat.m_hit_streak >= hdl->min_hits || hdl->frame_count <= hdl->min_hits)) { //未到hit数的也会输出
                out->item[tmp_result_num].b_match = 1;
            } else {
                out->item[tmp_result_num].b_match = 0;
            }
            tmp_result_num++;
        }
    }
    out->num = tmp_result_num;

    return 0;
}

int sort_tracker_init(void **phdl, SORT_PARAM *para) {
    SORT_HDL *hdl = malloc(sizeof(SORT_HDL));
    memset(hdl, 0, sizeof(SORT_HDL));

    if ((para == NULL) || (phdl == NULL)) {
        printf("ERR sort_tracker_init input NULL\n");
        return -1;
    }

    if (hdl == NULL) {
        printf("ERR sort_tracker_init malloc failed\n");
        return -1;
    }

    kalman_tracker_reset();

    hdl->max_age = para->max_age;
    hdl->min_hits = para->min_hits;
    hdl->iou_thrs = para->iou_thrs;
    hdl->iou_matrix = (float*)calloc(MAX_TRACKERS_NUM * MAX_TRACKERS_NUM, sizeof(float));

    *phdl = hdl;
    return 0;
}
