/**
 * @file tkl_audio.h
 * @brief 音频TKL层接口实现
 * @version 0.1
 * @date 2022-05-05
 * @copyright Copyright 2019-2021 Tuya Inc. All Rights Reserved.
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include "tkl_media.h"
#include "tkl_audio.h"
#include "tkl_detect_db.h"
#include "dsp/fh_common.h"
#include "dsp/fh_audio_mpi.h"
#include "tuya_iot_config.h"
#include "tuya_error_code.h"
#include "tkl_fh8636_audio.h"

#define FRAMENUM_IN_PKG (320) // APP_SYS_AV_AUDIO_FRAME_SIZE/2
#define TY_AUDIO_DB_CORRECT 10 // 开了电机降噪-10，另外本身计算的值偏低，加10修正
#define CHECK_RET(expr)                                                                                                \
    ret = expr;                                                                                                        \
    if (ret) {                                                                                                         \
        printf(#expr " failed ret = %08x\n", ret);                                                                     \
        return ret;                                                                                                    \
    } else {                                                                                                           \
        printf("done: " #expr "\n");                                                                                   \
    }

typedef struct {
    int enable;
    int isinit;
    int result;
} TY_MEDIA_CRY_HANDLE_T;

typedef struct {
    TY_MEDIA_PCM_HANDLE_T pcm_db_handle;
    TY_MEDIA_CRY_HANDLE_T pcm_cry_handle;
    unsigned char ai_ao_enable; // 0bit: ai enable, 1bit: ao enable

    FH_AC_CONFIG ai_param;
    int ai_micin_gain;
    unsigned int pcm_seq_no;
    int ai_lost_cnt; // 开启对讲时，ai ao sync enable 后，音频采集到异常波，丢掉一段音频
    int ai_init;
    int aec_flag;

    FH_AC_CONFIG ao_param;
    int ao_volume_offset;
    int ao_volume;
} TY_FH8636_AUDIO_HANDLE_T;

/**
 * @fn TY_FH8636_AUDIO_HANDLE_T* ty_fh8636_audio_get_handle()
 * @ 获取音频句柄
 * @param[in] 无
 * @retval 音频参数句柄
 * @attention 无。
 */
TY_FH8636_AUDIO_HANDLE_T *ty_fh8636_audio_get_handle()
{
    static TY_FH8636_AUDIO_HANDLE_T s_audio_handle;
    return &s_audio_handle;
}

/**
 * @fn int ty_fh8636_ai_init()
 * @ AI初始化
 * @param[in] 无
 * @retval 0成功 小于0失败
 * @attention 需要 将参数初始化后调用。
 */
OPERATE_RET tkl_ai_init(TKL_AUDIO_CONFIG_T *pconfig, int count)
{
    (void)count;
    int ret = -1;
    FH_UINT32 support_chmask = 0;
    FH_UINT32 version1 = 0;
    FH_UINT32 version2 = 0;
    FH_AC_MIX_CONFIG mix_config = {0};
    FH_AC_SesParam alg = {0};
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    memset(hand, 0x00, sizeof(TY_FH8636_AUDIO_HANDLE_T));
    hand->ai_micin_gain = 2; // micin gain

    // 配置富瀚mic参数
    hand->ai_param.io_type = FH_AC_MIC_IN; // FH_AC_LINE_IN (MIC_IN有功放)
    hand->ai_param.sample_rate = pconfig->sample;
    hand->ai_param.bit_width = pconfig->datebits;       //only support 16bit mode.
    hand->ai_param.enc_type = FH_PT_LPCM;               // only support raw PCM format
    hand->ai_param.channel_mask = pconfig->channel + 1; // only support one channel, channel_mask=1,使用 mic0
    hand->ai_param.period_size = FRAMENUM_IN_PKG;       // fh range: [80, 1024]

    CHECK_RET(FH_AC_Init());

    CHECK_RET(FH_AC_Get_Algo_Version(&version1, &version2));
    printf("[INFO]: Alg version(%d.%d.%d),%08x-%d\n", version2 >> 24, (version2 >> 16) & 0xff, (version2 >> 8) & 0xff,
           version2, version1);

    CHECK_RET(FH_AC_AI_Get_Feature_ChannelMask(&support_chmask));
    printf("[INFO]: supported AI channel_mask=%08x.\n", support_chmask);

    CHECK_RET(FH_AC_AI_Disable());
    hand->ai_ao_enable &= (~0x01);

    CHECK_RET(FH_AC_Set_Config(&hand->ai_param));

    mix_config.mix_enable = 1;
    mix_config.mix_channel_mask = 1;
    mix_config.reserved = 0;
    CHECK_RET(FH_AC_AI_MIX_Set_Config(&mix_config));

    CHECK_RET(tkl_ai_set_vol(0, 0, pconfig->mic_volume));

    ret = FH_AC_AI_MIX_Get_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AI_MIX_Get_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    alg.hpf_flag = 1;
    alg.anc_flag = 1;
    alg.agc_flag = 1;
    alg.doa_flag = 0;
    alg.aec_flag = 0;
    alg.alc_flag = 1;
    alg.alcParam.ALC_GAIN = -10;
    hand->aec_flag = 1; // 初始化状态，为ao start时，开启aec
    ret = FH_AC_AI_MIX_Set_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AI_MIX_Set_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    hand->ai_init = 1;
    return 0;
}

/***********************************************************
 *  函数名: tkl_ai_start
 *  功能：开启音频输入
 *  参数: Input，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_start(int card, TKL_AI_CHN_E chn)
{
    (void)card;
    // printf("tkl_ai_start card:%d chn:%d\n", card, chn);
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (chn != TKL_AI_0) {
        printf("tkl_ai_start chn = %d not sopport!\n", chn);
        return -1;
    }

    if (hand->ai_ao_enable & 0x02) {
        CHECK_RET(FH_AC_AO_Disable());
        CHECK_RET(FH_AC_AI_AO_SYNC_Enable());
    } else {
        CHECK_RET(FH_AC_AI_Enable());
    }
    hand->ai_ao_enable |= 0x01;
    return 0;
}

/***********************************************************
 *  函数名: tkl_ai_set_vol
 *  功能：设置音频ai 音量
 *  参数: Input，chn 通道号
 *        Input，vol 音量设置，取值[0, 100]
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_set_vol(int card, TKL_AI_CHN_E chn, int vol)
{
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    int volume = 0;
    int ret = -1;

    if (vol > 0) {
        volume = vol / 10 + 2;
    } else {
        volume = 0;
    }

    if (volume > 31) {
        printf("mic volume[%d] is out of range [0, 31], changed to max mic volume[31]\n", volume);
        volume = 31;
    }

    // FH_AC_AI_MICIN_SetVol(hand->ai_micin_gain) 是一级放大，ai_micin_gain = 3（26.4db）
    // FH_AC_AI_SetVol，这是第二级放大；取值范围 [0~31]，0 为静音最小，31 为最大 24db，步长：1.2dB
    // vol/10 + 5,取值：-12+5*1.2=-6dB， -12+15*1.2= 6dB
    printf("tkl_ai_set_vol card:%d, chn:%d, vol:%d, mic gain:%d, volume:%d\n", card, chn, vol, volume,
           hand->ai_micin_gain);
    CHECK_RET(FH_AC_AI_CH_SetAnologVol(0x0f, hand->ai_micin_gain, volume));
    return 0;
}

/***********************************************************
*  函数名: tkl_ai_get_frame
*  功能：获取音频帧数据
*  参数:  Input，chn，获取的码流通道
          Input Output:pframe,  pframe->pbuf和pframe->buf_size 入参，
              返回时，pbuf由底层拷贝数据，used_size 为实际数据长度
*
*  返回值: <0 失败，=0 成功
***********************************************************/
OPERATE_RET tkl_ai_get_frame(int card, TKL_AI_CHN_E chn, TKL_AUDIO_FRAME_INFO_T *pframe)
{
    (void)card;
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (chn != TKL_AI_0) {
        printf("type = %d is not support now !!!\n", chn);
        return -1;
    }

    if (hand->ai_init != 1) {
        return -1;
    }

    FH_AC_AI_Frame_S audio_frame = {0};
    ret = FH_AC_AI_GetFrame(&audio_frame); // 该接口不用分配内存
    // printf("ret = %d, len = %d\n", ret, audio_frame.len);
    if (ret == 0 && audio_frame.ch_data_len > 0) {
        if (hand->ai_lost_cnt > 0) {
            // printf("ai_lost_cnt: %d\n", hand->ai_lost_cnt);
            hand->ai_lost_cnt--;
            return -1;
        }

        if (audio_frame.ch_data_len > pframe->buf_size) {
            printf("audio bufSize[%d] isn't enough, must be large than %d!!!\n", pframe->buf_size,
                   audio_frame.ch_data_len);
            return -1;
        }
        pframe->codectype = TKL_CODEC_AUDIO_PCM;
        pframe->used_size = audio_frame.ch_data_len;
        pframe->timestamp = audio_frame.pts / 1000; // 富瀚是微妙级
        pframe->pts = pframe->timestamp;
        pframe->type = TKL_AUDIO_FRAME;
        pframe->seq = hand->pcm_seq_no++;
        pframe->sample = hand->ai_param.sample_rate;
        memcpy(pframe->pbuf, audio_frame.mix, audio_frame.ch_data_len);

        // pcm 传入音频检测模块
        if (hand->pcm_db_handle.enable && hand->pcm_db_handle.isinit) {
            if (hand->ai_ao_enable & 0x02) {
                hand->pcm_db_handle.pcmDb = 0;
            } else {
                ret = ty_media_pcm_db_set_data(&hand->pcm_db_handle, (char *)audio_frame.mix, audio_frame.ch_data_len);
                if (ret >= 0) {
                    hand->pcm_db_handle.pcmDb = ret + TY_AUDIO_DB_CORRECT;
                    // printf("ty_media_pcm_db get db:%d\n", hand->pcm_db_handle.pcmDb);
                }
            }
        }

        // pcm 传入哭声检测模块
        if (hand->pcm_cry_handle.enable && hand->pcm_cry_handle.isinit) {
            // 哭声16000,只支持单帧320
            if (hand->ai_param.sample_rate == 16000) {
                if (audio_frame.ch_data_len % 320 == 0) {
                    int cnt = audio_frame.ch_data_len / 320;
                    char *p = (char *)audio_frame.mix;
                    while (cnt > 0) {
                        ret = ty_audio_cry_send_data(p, 320);
                        if (ret < 0) {
                            printf("cry detect failed:%d\n", ret);
                        }
                        p = p + 320;
                        cnt--;
                    }
                } else {
                    printf("cry detect isnot support len:%d\n", audio_frame.ch_data_len);
                }
            } else {
                printf("cry detect isnot support\n");
            }
        }

        return 0;
    }

    printf("FH_AC_AI_GetFrame failed, ret = %0x\n", ret);
    return -1;
}

OPERATE_RET tkl_ai_set_vqe(INT32_T card, TKL_AI_CHN_E chn, TKL_AUDIO_VQE_TYPE_E type, TKL_AUDIO_VQE_PARAM_T *pparam)
{
    (void)card;
    int ret = 0;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    FH_AC_SesParam alg = {0};
    if (chn != TKL_AI_0) {
        printf("type = %d is not support now !!!\n", chn);
        return -1;
    }

    if (NULL == pparam) {
        printf("tkl_ai_set_vqe[%d] type = %d wrong, param is NULL\n", chn, type);
        return -1;
    }

    if (pparam->enable != 0 && pparam->enable != 1) {
        printf("tkl_ai_set_vqe[%d] type = %d enable %d  wrong\n", chn, type, pparam->enable);
        return -1;
    }

    ret = FH_AC_AI_MIX_Get_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AI_MIX_Get_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    printf("tkl_ai_set_vqe[%d] type  %d enable %d\n", chn, type, pparam->enable);
    switch (type) {
    case TKL_AUDIO_VQE_AEC:
        alg.aec_flag = pparam->enable;
        hand->aec_flag = alg.aec_flag;
        break;
    case TKL_AUDIO_VQE_NR:
        alg.anc_flag = pparam->enable;
        break;
    case TKL_AUDIO_VQE_AGC:
        alg.agc_flag = pparam->enable;
        break;
    case TKL_AUDIO_VQE_HPF:
        alg.hpf_flag = pparam->enable;
        break;
    case TKL_AUDIO_VQE_ALC:
        alg.alc_flag = pparam->enable;
        break;
    default:
        printf("tkl_ai_set_vqe type:%d, not support\n", type);
        return -1;
    }

    ret = FH_AC_AI_MIX_Set_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AI_MIX_Set_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    return 0;
}

OPERATE_RET tkl_ai_get_vqe(INT32_T card, TKL_AI_CHN_E chn, TKL_AUDIO_VQE_TYPE_E type, TKL_AUDIO_VQE_PARAM_T *pparam)
{
    (void)card;
    FH_AC_SesParam alg = {0};
    int ret = -1;

    if (chn != TKL_AI_0) {
        printf("tkl_ai_get_vqe type = %d is not support now !!!\n", chn);
        return -1;
    }

    if (NULL == pparam) {
        printf("tkl_ai_get_vqe[%d] type = %d wrong, param is NULL\n", chn, type);
        return -1;
    }

    ret = FH_AC_AI_MIX_Get_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AI_MIX_Get_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    switch (type) {
    case TKL_AUDIO_VQE_AEC:
        pparam->enable = alg.aec_flag;
        break;
    case TKL_AUDIO_VQE_NR:
        pparam->enable = alg.anc_flag;
        break;
    case TKL_AUDIO_VQE_AGC:
        pparam->enable = alg.agc_flag;
        break;
    case TKL_AUDIO_VQE_HPF:
        pparam->enable = alg.hpf_flag;
        break;
    case TKL_AUDIO_VQE_ALC:
        pparam->enable = alg.alc_flag;
        break;
    default:
        printf("tkl_ai_get_vqe type:%d, not support\n", type);
        return -1;
    }
    printf("tkl_ai_get_vqe[%d] type  %d enable %d\n", chn, type, pparam->enable);
    return 0;
}

/***********************************************************
 *  函数名: tkl_ai_stop
 *  功能：停止音频输入
 *  参数: Input，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_stop(int card, TKL_AI_CHN_E chn)
{
    int ret = -1;
    printf("tkl_ai_stop card:%d, chn:%d\n", card, chn);
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (chn != TKL_AI_0) {
        printf("tkl_ai_stop chn = %d not sopport!\n", chn);
        return -1;
    }

    CHECK_RET(FH_AC_AI_Disable());
    hand->ai_ao_enable &= (~0x01);
    return 0;
}

/***********************************************************
 *  函数名: tkl_ai_uninit
 *  功能：去初始化
 *  参数: Input，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_uninit(void)
{
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    hand->ai_init = 0;
    FH_AC_DeInit();
    if (hand->pcm_cry_handle.isinit == 1) {
        hand->pcm_cry_handle.isinit = 0;
        ty_audio_cry_uninit();
    }

    if (hand->pcm_db_handle.isinit == 1) {
        hand->pcm_db_handle.isinit = 0;
        ty_media_pcm_db_uninit(&hand->pcm_db_handle);
    }

    return 0;
}

/***********************************************************
 *  函数名: tkl_ao_init
 *  功能：audio out 初始化函数
 *  参数: Input TKL_AUDIO_CONFIG *pconfig，音频ao的初始化参数
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ao_init(TKL_AUDIO_CONFIG_T *pconfig, INT32_T count, VOID **handle)
{
    (void)handle;
    (void)count;
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    FH_AC_SesParam alg = {0};

    // 配置富瀚spk参数
    hand->ao_param.io_type = FH_AC_LINE_OUT;
    hand->ao_param.sample_rate = pconfig->sample;
    hand->ao_param.bit_width = pconfig->datebits;       // only support 16bit mode.
    hand->ao_param.enc_type = FH_PT_LPCM;               // only support raw PCM format
    hand->ao_param.channel_mask = pconfig->channel + 1; // only support one channel
    hand->ao_param.period_size = FRAMENUM_IN_PKG;       // fh range: [80, 1024]
    hand->ao_param.frame_num = 0;                       // default 0,mean use max buffer.
    hand->ao_param.reserved = 0;
    hand->ao_volume_offset = pconfig->spk_volume_offset;
    CHECK_RET(FH_AC_AO_Disable());
    hand->ai_ao_enable &= (~0x02);

    CHECK_RET(FH_AC_Set_Config(&hand->ao_param));
    CHECK_RET(tkl_ao_set_vol(0, 0, NULL, pconfig->spk_volume));

    ret = FH_AC_AO_Get_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AO_Get_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }
    alg.hpf_flag = 0;
    alg.anc_flag = 0;
    alg.agc_flag = 0;
    ret = FH_AC_AO_Set_Algo_Param(&alg, sizeof(alg));
    if (ret) {
        printf("FH_AC_AO_Set_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }

    hand->pcm_db_handle.pcmDb = -1;

    return ret;
}

aecParam aec_param = {
    .BNG_FLAG = 1,      // background noise generation switch
    .AEC_DELAY_MS = 45, // echo delay in ms
    .AEC_ECSTART_THR = 150,
    .AEC_MIU = 165,
    .NLP_LEVEL = 0x0004,
    .NLP_GAMMA = 13746,
    .POST_FILTER_SWITCH = 4,
    .ECHO_ON_FR_SWITCH = 1,
    .ERLE_THR1 = 2000,
    .ERLE_SUBTHR1 = 9638,
    .ERLE_THR2 = 3000,
    .ERLE_SUBTHR2 = 8300,
    .POS_S = 0,
    .POS_BIT = 4,
    .REVERBERATION = 10,
    .REV_SMOOTH = 16300,
    .TX_RX_RATIO = {1, 10, 1850, 3800, 3500, 3500, 1500, 1500, 1500, 1500, 1500,
                    1500}, // Q10{ 0,1,2,4,8,16,24,32,48,64,96,128 }
    .DTD_SWITCH = 0,
    .DTD_THR = 10000,
    .DTD_GAIN = 1,
    .NON_DTD_GAIN = 2,
};

static int __set_aec(int enable)
{
    FH_AC_SesParam alg = {0};
    int ret = FH_AC_AI_MIX_Get_Algo_Param(&alg, sizeof(alg));
    if (ret != 0) {
        printf("FH_AC_AI_MIX_Get_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }
    alg.aec_flag = enable;
    alg.aecParam = aec_param;
#if 0
    printf("alg.aecParam.BNG_FLAG %d, alg.aecParam.AEC_DELAY_MS %d, alg.aecParam.AEC_ECSTART_THR %d, "
           "alg.aecParam.AEC_MIU %d\n",
           alg.aecParam.BNG_FLAG, alg.aecParam.AEC_DELAY_MS, alg.aecParam.AEC_ECSTART_THR, alg.aecParam.AEC_MIU);
    printf("alg.aecParam.NLP_LEVEL %d, alg.aecParam.NLP_GAMMA %d, alg.aecParam.POST_FILTER_SWITCH %d\n",
           alg.aecParam.NLP_LEVEL, alg.aecParam.NLP_GAMMA, alg.aecParam.POST_FILTER_SWITCH);
    printf("alg.aecParam.ECHO_ON_FR_SWITCH %d, alg.aecParam.ERLE_THR1 %d,alg.aecParam.ERLE_SUBTHR1 %d, "
           "alg.aecParam.ERLE_THR2 %d\n",
           alg.aecParam.ECHO_ON_FR_SWITCH, alg.aecParam.ERLE_THR1, alg.aecParam.ERLE_SUBTHR1, alg.aecParam.ERLE_THR2);
    printf(
        "alg.aecParam.ERLE_SUBTHR2 %d, alg.aecParam.POS_S %d, alg.aecParam.POS_BIT %d, alg.aecParam.REVERBERATION %d\n",
        alg.aecParam.ERLE_SUBTHR2, alg.aecParam.POS_S, alg.aecParam.POS_BIT, alg.aecParam.REVERBERATION);
    printf("alg.aecParam.REV_SMOOTH %d, alg.aecParam.DTD_SWITCH %d, alg.aecParam.DTD_THR %d\n", alg.aecParam.REV_SMOOTH,
           alg.aecParam.DTD_SWITCH, alg.aecParam.DTD_THR);
    printf("alg.aecParam.DTD_GAIN %d, alg.aecParam.NON_DTD_GAIN %d\n", alg.aecParam.DTD_GAIN,
           alg.aecParam.NON_DTD_GAIN);
#endif
    ret = FH_AC_AI_MIX_Set_Algo_Param(&alg, sizeof(alg));
    if (ret != 0) {
        printf("FH_AC_AI_MIX_Set_Algo_Param: error, ret=%d!\n", ret);
        return -1;
    }
    return ret;
}

/***********************************************************
 *  函数名: tkl_ao_start
 *  功能：开启音频输出
 *  参数: Input，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ao_start(INT32_T card, TKL_AO_CHN_E chn, VOID *handle)
{
    (void)handle;
    printf("tkl_ao_start card:%d, chn:%d\n", card, chn);
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (chn != TKL_AO_0) {
        printf("tkl_ao_start chn = %d not sopport!\n", chn);
        return -1;
    }

    if (hand->ai_ao_enable & 0x01) {
        hand->ai_lost_cnt = 3;
        if (hand->aec_flag == 1) {
            __set_aec(hand->aec_flag);
        }
        CHECK_RET(FH_AC_AI_Disable());
        CHECK_RET(FH_AC_AI_AO_SYNC_Enable()); // for AEC, must use this to enable AI and AO the same time.
    } else {
        CHECK_RET(FH_AC_AO_Enable());
    }
    hand->ai_ao_enable |= 0x02;
    return 0;
}

/***********************************************************
 *  函数名: tkl_ao_set_vol
 *  功能：设置ao音量
 *  参数: Input，chn 通道号
 *        Input，vol, 音量[0,100]
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ao_set_vol(int card, TKL_AO_CHN_E chn, void *handle, int vol)
{
    (void)card;
    (void)chn;
    (void)handle;
    int ao_vol = 0;
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();

    ao_vol = vol * (100 + hand->ao_volume_offset) * 3 / 4 / 100; // 由原来的取值0~100 改为0~75
    if (ao_vol < 0) {
        ao_vol = 0;
    } else if (ao_vol > 128) {
        ao_vol = 128;
    }
    CHECK_RET(FH_AC_AO_SetDigitalVol(ao_vol)); // 0~128
    hand->ao_volume = ao_vol;
    printf("ao_vol:%d, offset:%d, set:%d\n", vol, hand->ao_volume_offset, ao_vol);
    return 0;
}

OPERATE_RET tkl_ao_get_vol(INT32_T card, TKL_AO_CHN_E chn, VOID *handle, INT32_T *vol)
{
    (void)card;
    (void)chn;
    (void)handle;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    *vol = hand->ao_volume;
    printf("tkl_ao_get_vol vol:%d\n", *vol);
    return 0;
}

/***********************************************************
*  函数名: tkl_ao_put_frame
*  功能： 播放音频帧数据
*  参数:  Input，chn，码流通道
          Input，TKL_AUDIO_FRAME *pframe，音频帧数据
*
*  返回值: <0 失败，=0 成功
***********************************************************/
OPERATE_RET tkl_ao_put_frame(int card, TKL_AO_CHN_E chn, void *handle, TKL_AUDIO_FRAME_INFO_T *pframe)
{
    (void)card;
    (void)chn;
    (void)handle;
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();

    if (TKL_AUDIO_FRAME != pframe->type) {
        printf("type = %d not sopport!\n", pframe->type);
        return -1;
    }

    if (TKL_CODEC_AUDIO_PCM == pframe->codectype) {
        if ((uint32_t)hand->ao_param.sample_rate != pframe->sample) {
            printf("wrong frame sample_rate %d! want %d\n", pframe->type, hand->ao_param.sample_rate);
            return -1;
        }

        FH_AC_AO_FRAME_S audio_frame = {0};
        audio_frame.data = (uint8_t *)pframe->pbuf;
        audio_frame.len = pframe->used_size;
        ret = FH_AC_AO_SendFrame(&audio_frame);
        if (ret != 0) {
            printf("FH_AC_AO_SendFrame ret = %08x\n", ret);
            return -1;
        }
    } else {
        printf("codectype = %d not sopport!\n", pframe->codectype);
        return -1;
    }
    return 0;
}

/***********************************************************
 *  函数名: tkl_ao_stop
 *  功能：关闭ao
 *  参数: Input，chn 通道号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ao_stop(int card, TKL_AO_CHN_E chn, void *handle)
{
    (void)handle;
    (void)card;
    // printf("tkl_ao_stop card:%d, chn:%d\n", card, chn);
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (chn != TKL_AO_0) {
        printf("chn = %d not sopport!\n", chn);
        return -1;
    }

    CHECK_RET(FH_AC_AO_Disable());
    __set_aec(0); // 文件播放结束,关闭回声消除
    hand->ai_ao_enable &= (~0x02);

    return 0;
}

/***********************************************************
 *  函数名: tkl_ao_uninit
 *  功能：去初始化
 *  参数: Input，handle
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ao_uninit(VOID *handle)
{
    (void)handle;
    return 0;
}

/***********************************************************
 *  函数名: __ai_detect_db_start
 *  功能：开启音频分贝检测
 *  参数: Input，card 声卡号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
static OPERATE_RET __ai_detect_db_start(int card)
{
    (void)card;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (hand->pcm_db_handle.enable) {
        printf("audio detect is already enable!\n");
        return 0;
    }

    TY_MEDIA_PCM_PARAM_T pcm_db_param = {0};
    if (hand->pcm_db_handle.isinit == 0) {
        // 初始化声音检测
        pcm_db_param.channel = 1;
        pcm_db_param.sampleRate = hand->ai_param.sample_rate;
        pcm_db_param.bitRate = hand->ai_param.bit_width;
        pcm_db_param.sampleCnt = pcm_db_param.sampleRate / 5; // 1/5s
        ty_media_pcm_db_init(&hand->pcm_db_handle, &pcm_db_param);
        hand->pcm_db_handle.isinit = 1;
    }

    hand->pcm_db_handle.enable = 1;
    printf("tkl_ai_detect_db_start\n");
    return 0;
}

/***********************************************************
 *  函数名: __ai_detect_db_stop
 *  功能：关闭音频分贝检测
 *  参数: Input，card 声卡号
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
static OPERATE_RET __ai_detect_db_stop(int card)
{
    (void)card;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    hand->pcm_db_handle.enable = 0;
    return 0;
}

/**
 *
 * @fn __ai_detect_db_get_result(int card, TKL_AUDIO_DETECT_DB_RESULT_S *pResult)
 * @ 获取分贝检测结果
 * @param[in] Input，card 声卡号，
 * @param[out] pResult 检测结果
 * @retval 0成功 小于0失败
 * @attention 无。
 */
static OPERATE_RET __ai_detect_db_get_result(int card, TKL_AUDIO_DETECT_DB_RESULT_T *pResult)
{
    (void)card;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    if (hand->pcm_db_handle.isinit == 0 || hand->pcm_db_handle.enable == 0) {
        printf("audio_detect init:%d, enable:%d\n", hand->pcm_db_handle.isinit, hand->pcm_db_handle.enable);
        return -1;
    }
    pResult->pcm_db = hand->pcm_db_handle.pcmDb;
    return 0;
}

static OPERATE_RET __ai_detect_cry_start(int card)
{
    (void)card;
    int ret = -1;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    printf("tkl_ai_detect_cry_start start\n");
    if (hand->pcm_cry_handle.enable) {
        printf("audio detect is already enable!\n");
        return 0;
    }

    if (hand->pcm_cry_handle.isinit == 0) {
        // 初始化声音检测
        ret = ty_audio_cry_init(hand->ai_param.sample_rate);
        if (ret != 0) {
            printf("ty_audio_cry_init failed!\n");
            return -1;
        }
        hand->pcm_cry_handle.isinit = 1;
    }

    hand->pcm_cry_handle.enable = 1;
    printf("tkl_ai_detect_cry_start end\n");

    return 0;
}

static OPERATE_RET __ai_detect_cry_stop(int card)
{
    (void)card;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    printf("tkl_ai_detect_cry_stop\n");
    hand->pcm_cry_handle.enable = 0;
    return 0;
}

/***********************************************************
 *  函数名: __ai_detect_cry_get_result
 *  功能：获取哭声检测结果
 *  参数: Input，card 声卡号
 *        Output，pResult 结果
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
static OPERATE_RET __ai_detect_cry_get_result(int card, TKL_AUDIO_DETECT_RESULT_T *pResult)
{
    (void)card;
    TY_FH8636_AUDIO_HANDLE_T *hand = ty_fh8636_audio_get_handle();
    int val = 0;
    int ret = -1;

    memset(pResult, 0, sizeof(TKL_AUDIO_DETECT_RESULT_T));
    if (hand->pcm_cry_handle.isinit == 0 || hand->pcm_cry_handle.enable == 0) {
        printf("audio_detect not start, init:%d, enable:%d\n", hand->pcm_cry_handle.isinit,
               hand->pcm_cry_handle.enable);
        return -1;
    }

    ret = ty_audio_cry_get_result(&val);
    if (ret < 0) {
        printf("ty_audio_cry_get_result failed:%d\n", ret);
        return -1;
    }
    pResult->val = val;
    return 0;
}

/***********************************************************
 *  函数名: tkl_ai_detect_start
 *  功能：开启音频检测
 *  参数: Input，card 声卡号
 *        Input，type 检测类型
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_detect_start(INT32_T card, TKL_MEDIA_DETECT_TYPE_E type)
{
    (void)card;
    int ret = -1;
    switch (type) {
    case TKL_MEDIA_DETECT_TYPE_BABY_CRY:
        ret = __ai_detect_cry_start(card);
        break;

    case TKL_MEDIA_DETECT_TYPE_DB:
        ret = __ai_detect_db_start(card);
        break;

    default:
        printf("tkl_ai_detect_start type:%d, not supported\n", type);
        break;
    }
    return ret;
}

/***********************************************************
 *  函数名: tkl_ai_detect_stop
 *  功能：关闭音频检测
 *  参数: Input，card 声卡号
 *        Input，type 检测类型
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_detect_stop(INT32_T card, TKL_MEDIA_DETECT_TYPE_E type)
{
    (void)card;
    int ret = -1;
    switch (type) {
    case TKL_MEDIA_DETECT_TYPE_BABY_CRY:
        ret = __ai_detect_cry_stop(card);
        break;

    case TKL_MEDIA_DETECT_TYPE_DB:
        ret = __ai_detect_db_stop(card);
        break;

    default:
        printf("tkl_ai_detect_start type:%d, not supported\n", type);
        break;
    }
    return ret;
}

/***********************************************************
 *  函数名: tkl_ai_detect_get_result
 *  功能：获取音频检测结果
 *  参数: Input，card 声卡号
 *        Input，type 检测类型
 *        Output，pResult 结果
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_ai_detect_get_result(INT32_T card, TKL_MEDIA_DETECT_TYPE_E type, TKL_AUDIO_DETECT_RESULT_T *presult)
{
    (void)card;
    int ret = -1;
    TKL_AUDIO_DETECT_DB_RESULT_T db = {0};
    if (presult == NULL) {
        printf("presult is NULL\n");
        return -1;
    }

    switch (type) {
    case TKL_MEDIA_DETECT_TYPE_BABY_CRY:
        ret = __ai_detect_cry_get_result(card, presult);
        break;

    case TKL_MEDIA_DETECT_TYPE_DB:
        ret = __ai_detect_db_get_result(card, &db);
        presult->val = db.pcm_db;
        break;

    default:
        printf("tkl_ai_detect_start type:%d, not supported\n", type);
        break;
    }

    return ret;
}
