/**
 * @file tkl_video_enc.c
 * @brief Common process - implement the tkl_video_enc.c
 * @version 0.1
 * @date 2021-08-06
 *
 * @copyright Copyright 2018-2021 Tuya Inc. All Rights Reserved.
 *
 */
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/prctl.h>

#include "tkl_video_in.h"
#include "tkl_video_enc.h"
#include "tkl_fh8636_video.h"
#include "tkl_fh8636_osd.h"
#include "tkl_fh8636_hd.h"

// 原厂sdk头文件
#include "isp_api.h"
// #include "FHAdv_Isp_mpi_v2.h"
#include "fh_system_mpi.h"
#include "fh_vpu_mpi.h"
#include "mpp/fh_vb_mpi.h"
#include "fh_venc_mpi.h"
#include "dbi/debug.h"

// vpu 通道
#define VPU_CHN0     0
#define VPU_CHN1     1
#define VPU_CHN2     2
#define VPU_CHN3     3
#define VPU_CHN_JPEG VPU_CHN1

// venc 通道
#define VENC_CHN_MAIN 0
#define VENC_CHN_SUB  1
#define VENC_CHN_JPEG 2

#define H264_BREATH_IP_QPDELTA 3
#define H264_BREATH_I_BITPROP  5
#define H264_BREATH_P_BITPROP  1

#define H265_BREATH_IP_QPDELTA 3
#define H265_BREATH_I_BITPROP  5
#define H265_BREATH_P_BITPROP  1

#define THRD_DSP_VIDEO "dsp_video"

#define ALIGNTO(addr, edge) ((addr + edge - 1) & ~(edge - 1)) // 数据结构对齐定义

#ifndef FH_APP_YUV_CHANNEL
#define FH_APP_YUV_CHANNEL VPU_CHN2 // yuv channel (QR and MD)
#endif

#define MAX_FRAME_BUFFER_SIZE (300 * 1024)
#define SEI_FRAME_BUFFER_SIZE (1024)                                 // 给SEI 帧预留1024字节
#define MAX_I_FRAME_BITS      (MAX_FRAME_BUFFER_SIZE * 8 * 85 / 100) // 最大I帧大小定为85%缓存大小

#define ISP_PROC   "/proc/driver/isp"
#define VPU_PROC   "/proc/driver/vpu"
#define BGM_PROC   "/proc/driver/bgm"
#define ENC_PROC   "/proc/driver/enc"
#define JPEG_PROC  "/proc/driver/jpeg"
#define TRACE_PROC "/proc/driver/trace"

#define WR_PROC_DEV(device, cmd)                                                                                       \
    do {                                                                                                               \
        int _tmp_fd;                                                                                                   \
        _tmp_fd = open(device, O_RDWR, 0);                                                                             \
        if (_tmp_fd >= 0) {                                                                                            \
            printf("w proc:device:%s,cmd:%s\n", device, cmd);                                                          \
            write(_tmp_fd, cmd, sizeof(cmd));                                                                          \
            close(_tmp_fd);                                                                                            \
        }                                                                                                              \
    } while (0)

typedef struct {
    int srcWidth;
    int srcHeight;
    int dstWidth;
    int dstHeight;
} TY_VIDEO_SCALE_PATA_T;

static TY_FH8636_VIDEO_HANDLE_T g_video_handle = {0}; // 视频的流的句柄
static void *fh8636_venc_proc(void *arg);
int32_t fh8636_video_param_init(TKL_VENC_CONFIG_T *pconfig, int count);
int32_t fh8636_start();

/**
 * @fn TY_FH8636_VIDEO_HANDLE_T * __get_video_handle()
 * @获取视频句柄
 * @param[in] 无
 * @retval 视频参数句柄指针
 * @attention 无。
 */
TY_FH8636_VIDEO_HANDLE_T *__get_video_handle()
{
    return &g_video_handle;
}

/***********************************************************
*  函数名: tkl_venc_init
*  功能：初始化函数
*  参数: Input，vi_chn,vi 的chn号，多目预留
        Input，pconfig，初始化参数
*
*  返回值: <0 失败，=0 成功
***********************************************************/
OPERATE_RET tkl_venc_init(int vi_chn, TKL_VENC_CONFIG_T *pconfig, int count)
{
    (void)vi_chn;
    int ret = 0;
    TY_FH8636_VIDEO_HANDLE_T *hand = __get_video_handle();
    if (hand->isInit == 1) {
        printf("tkl_venc_init already initialized\n");
        return 0;
    }

    // 参数初始化
    ret = fh8636_video_param_init(pconfig, count);
    TY_CHECK_RETURN(ret, -1);

    // 富瀚dsp&isp初始化
    ret = fh8636_start();
    TY_CHECK_RETURN(ret, -1);

    printf("tkl_venc_init success!!!\n");
    hand->isInit = 1;

    // 初始化 yuv 锁
    ret = pthread_mutex_init(&hand->yuv_mutex, NULL);
    TY_CHECK_RETURN(ret, -1);

    // 初始化 venc 锁
    ret = pthread_mutex_init(&hand->enc_mutex, NULL);
    TY_CHECK_RETURN(ret, -1);
    ret = tkl_venc_start(0, 0);
    TY_CHECK_RETURN(ret, -1);

    ret = tkl_venc_start(0, 1);
    TY_CHECK_RETURN(ret, -1);

    if (hand->vpuParam[VPU_CHN2].enable == 1) {
        ret = ty_fh8636_hd_init();
        TY_CHECK_RETURN(ret, -1);
    }
#ifdef FH_APP_USING_COOLVIEW
    printf("create coolview thread\n");
    sample_common_start_coolview();
#endif

    return 0;
}

/***********************************************************
*  函数名: tkl_venc_set_osd
*  功能：osd 属性设置
*  参数: Input，vi_chn，多目预留
        Input，venc_chn，编码通道
        Input，posd, osd 参数
*
*  返回值: <0 失败，=0 成功
***********************************************************/
OPERATE_RET tkl_venc_set_osd(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_OSD_T *posd)
{
    (void)vi_chn;
    (void)venc_chn;
    printf("tkl_venc_set_osd is dls:%d, enable:%d\n", posd->is_dls, posd->enable);
    if (posd->enable == 1) {
        fh8636_osd_enable(true);
    } else {
        fh8636_osd_enable(false);
    }
    return 0;
}

/**
 * @brief video encode request IDR frame
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc channel number
 * @param[in] idr_type: request idr type
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_venc_set_IDR(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_IDR_E idr_type)
{
    // --- BEGIN: user implements ---
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    if (vi_chn != 0) {
        printf("tkl_venc_set_IDR wrong vi_chn = %d\n", vi_chn);
        return -1;
    }
    if (TKL_VENC_0 != venc_chn && TKL_VENC_1 != venc_chn) {
        printf("tkl_venc_set_IDR type = %d, not need!\n", venc_chn);
        return 0;
    }
    printf("tkl_venc_set_IDR idr_type:%d\n", idr_type);
    if (idr_type == TKL_VENC_IDR_REQUEST) {
        int ret = FH_VENC_RequestIDR(pHand->encParam[venc_chn].encChn);
        if (ret != RETURN_OK) {
            printf("FH_VENC_RequestIDR failed, ret = %08x\n", ret);
            return -1;
        }
    } else {
        printf("tkl_venc_set_IDR not supported idr_type:%d", idr_type);
    }
    return 0;
    // --- END: user implements ---
}

/**
 * @brief video encode set mask
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc channel number
 * @param[out] pmask: mask config
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_venc_set_mask(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_MASK_T *pmask)
{
    (void)vi_chn;
    (void)venc_chn;
    (void)pmask;
    printf("tkl_venc_set_mask not support\n");
    return -1;
}

/**
 * @brief video first snap
 *
 * @param[in] vi_chn: vi channel number
 * @param[out] pframe: output frame
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_venc_get_first_snap(TKL_VI_CHN_E vi_chn, TKL_VENC_FRAME_T *pframe)
{
    (void)vi_chn;
    (void)pframe;
    printf("not support\n");
    return -1;
}

/***********************************************************
 *  函数名: tkl_venc_start
 *  功能：venc start
 *  参数: Input，TKL_VI_CHN_E vi_chn
 *               TKL_VENC_CHN_E venc_chn
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_venc_start(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn)
{
    pthread_t tid = 0;
    TY_FH8636_VIDEO_HANDLE_T *hand = __get_video_handle();
    if (vi_chn != 0) {
        printf("ty_dev_venc_start wrong vi_chn = %d\n", vi_chn);
        return -1;
    }

    if (TKL_VENC_0 != venc_chn && TKL_VENC_1 != venc_chn) {
        printf("ty_dev_venc_start type = %d, not need!\n", venc_chn);
        return 0;
    }
    if (hand->encParam[venc_chn].trans_mode == TKL_VENC_PUSH_FRAME_BY_CALLBACK) {

        // 创建取流线程
        if (hand->thread_status) {
            return 0;
        }
        hand->frame_buf = malloc(MAX_FRAME_BUFFER_SIZE);
        if (hand->frame_buf == NULL) {
            printf("malloc tmp stream_buf failed\n");
            return -1;
        }
        memset(hand->frame_buf, 0, MAX_FRAME_BUFFER_SIZE);
        hand->thread_status = 1;
        if (0 > pthread_create(&tid, NULL, fh8636_venc_proc, NULL)) {
            hand->thread_status = 0;
            printf("pthread_create\n");
            return -1;
        }
        pthread_detach(tid);
    }

    printf("ty_dev_venc_start venc_chn = %d success\n", venc_chn);
    return 0;
}

/***********************************************************
 *  函数名: tkl_venc_stop
 *  功能：venc stop
 *  参数: Input，TKL_VI_CHN_E vi_chn
 *               TKL_VENC_CHN_E venc_chn
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_venc_stop(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn)
{
    (void)vi_chn;
    (void)venc_chn;
    return 0;
}

/***********************************************************
 *  函数名: tkl_venc_uninit
 *  功能：去初始化
 *  参数: Input，TKL_VI_CHN_E vi_chn
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_venc_uninit(TKL_VI_CHN_E vi_chn)
{
#ifdef FH_APP_USING_COOLVIEW
    sample_common_stop_coolview();
    printf("=====sample_common_stop_coolview=====\n");
#endif

    (void)vi_chn;
    return 0;
}

/**
 * @brief video encode stream buff pool set
 *
 * @param[in] vi_chn: vi channel number
 * @param[in] venc_chn: venc channel number
 * @param[in] parg:  buff pool config
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_venc_set_video_stream_buffer(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_STREAM_BUFF_T *parg)
{
    (void)vi_chn;
    (void)venc_chn;
    (void)parg;
    return OPRT_OK;
}

int32_t fh8636_video_param_init(TKL_VENC_CONFIG_T *pconfig, int count)
{
    int32_t i = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    if (count < 1 || pconfig == NULL) {
        printf("config:%d, pconfig:%p\n", count, pconfig);
        return -1;
    }

    for (i = 0; i < count; i++) {
        printf("i:%d, w,h:%d %d\n", i, pconfig[i].width, pconfig[i].height);
        if (pconfig[i].enable == 0) {
            printf("i:%d, type:%d enable:%d,continue\n", i, pconfig[i].type, pconfig[i].enable);
            continue;
        }
        // 参数配置
        pHand->encParam[i].width = pconfig[i].width;
        pHand->encParam[i].height = pconfig[i].height;
        if (TKL_CODEC_VIDEO_H265 == pconfig[i].codectype) {
            pHand->encParam[i].vencCfg.chn_attr.enc_type = FH_NORMAL_H265;
            pHand->encParam[i].vencCfg.chn_attr.h265_attr.profile = H265_PROFILE_MAIN;
            pHand->encParam[i].vencCfg.chn_attr.h265_attr.i_frame_intterval = pconfig[i].gop;
            pHand->encParam[i].vencCfg.chn_attr.h265_attr.size.u32Width = pconfig[i].width;
            pHand->encParam[i].vencCfg.chn_attr.h265_attr.size.u32Height = pconfig[i].height;
            pHand->encParam[i].vencCfg.rc_attr.rc_type = FH_RC_H265_VBR;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.bitrate = pconfig[i].bitrate * 1000 * 7 / 8;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.init_qp = 35;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.ImaxQP = pconfig[i].max_qp;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.IminQP = pconfig[i].min_qp + 3;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.PmaxQP = pconfig[i].max_qp;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.PminQP = pconfig[i].min_qp + 3;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.FrameRate.frame_count = pconfig[i].fps;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.FrameRate.frame_time = 1;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.maxrate_percent = 120;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.IFrmMaxBits = MAX_I_FRAME_BITS;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.IP_QPDelta = H265_BREATH_IP_QPDELTA;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.I_BitProp = H265_BREATH_I_BITPROP;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.P_BitProp = H265_BREATH_P_BITPROP;
            pHand->encParam[i].vencCfg.rc_attr.h265_vbr.fluctuate_level = 0;
        } else if (TKL_CODEC_VIDEO_H264 == pconfig[i].codectype) {
            pHand->encParam[i].vencCfg.chn_attr.enc_type = FH_NORMAL_H264;
            pHand->encParam[i].vencCfg.chn_attr.h264_attr.profile = H264_PROFILE_BASELINE;
            pHand->encParam[i].vencCfg.chn_attr.h264_attr.i_frame_intterval = pconfig[i].gop;
            pHand->encParam[i].vencCfg.chn_attr.h264_attr.size.u32Width = pconfig[i].width;
            pHand->encParam[i].vencCfg.chn_attr.h264_attr.size.u32Height = pconfig[i].height;
            pHand->encParam[i].vencCfg.rc_attr.rc_type = FH_RC_H264_VBR;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.bitrate = pconfig[i].bitrate * 1000;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.init_qp = 35;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.ImaxQP = pconfig[i].max_qp;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.IminQP = pconfig[i].min_qp;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.PmaxQP = pconfig[i].max_qp;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.PminQP = pconfig[i].min_qp;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.FrameRate.frame_count = pconfig[i].fps;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.FrameRate.frame_time = 1;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.maxrate_percent = 200;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.IFrmMaxBits = MAX_I_FRAME_BITS;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.IP_QPDelta = H264_BREATH_IP_QPDELTA;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.I_BitProp = H264_BREATH_I_BITPROP;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.P_BitProp = H264_BREATH_P_BITPROP;
            pHand->encParam[i].vencCfg.rc_attr.h264_vbr.fluctuate_level = 0;
        } else if (TKL_CODEC_VIDEO_MJPEG == pconfig[i].codectype) {
            pHand->encParam[i].vencCfg.chn_attr.enc_type = FH_JPEG;
            pHand->encParam[i].vencCfg.chn_attr.jpeg_attr.qp = pconfig[i].max_qp;
            pHand->encParam[i].vencCfg.chn_attr.jpeg_attr.rotate = 0;
            pHand->encParam[i].vencCfg.chn_attr.jpeg_attr.encode_speed = 4; /* 大幅面下编码器性能不够时，设置为0 */
        } else {
            pHand->encParam[i].vencCfg.rc_attr.rc_type = FH_RC_MJPEG_FIXQP;
            pHand->encParam[i].vencCfg.rc_attr.mjpeg_fixqp.qp = 50;
            pHand->encParam[i].vencCfg.rc_attr.mjpeg_fixqp.FrameRate.frame_count = pconfig[i].fps;
            pHand->encParam[i].vencCfg.rc_attr.mjpeg_fixqp.FrameRate.frame_time = 1;
        }
        if (i == TKL_VENC_SNAP) {
            pHand->encParam[i].vencCap.support_type = FH_JPEG;
        } else {
            pHand->encParam[i].vencCap.support_type = FH_NORMAL_H264 | FH_NORMAL_H265;
        }
        pHand->encParam[i].vencCap.max_size.u32Height = pconfig[i].height;
        pHand->encParam[i].vencCap.max_size.u32Width = pconfig[i].width;

        pHand->format_flag[i] = 0;
        pHand->format[i].width = pconfig[i].width;
        pHand->format[i].height = pconfig[i].height;
        pHand->format[i].codectype = pconfig[i].codectype;
        if (TKL_CODEC_VIDEO_H264 == pconfig[i].codectype) {
            pHand->format[i].h264_attr.bitrate = pconfig[i].bitrate;
            pHand->format[i].h264_attr.fps = pconfig[i].fps;
            pHand->format[i].h264_attr.gop = pconfig[i].gop;
            pHand->format[i].h264_attr.profile = TKL_VENC_H264_PROFILE_BASELINE;
        } else if (TKL_CODEC_VIDEO_H265 == pconfig[i].codectype) {
            pHand->format[i].h265_attr.bitrate = pconfig[i].bitrate;
            pHand->format[i].h265_attr.fps = pconfig[i].fps;
            pHand->format[i].h265_attr.gop = pconfig[i].gop;
            pHand->format[i].h265_attr.profile = TKL_VENC_H265_PROFILE_MAIN;
        } else if (TKL_CODEC_VIDEO_MJPEG == pconfig[i].codectype) {
            pHand->format[i].jpeg_attr.qp = 50;
        }
        // printf("i:%d, type:%d enable:%d\n", i, pconfig[i].type, pconfig[i].enable);
        //  专有参数配置
        switch (pconfig[i].type) {
        case TKL_VENC_MAIN: {
            // vpu 参数
            pHand->vpuParam[VPU_CHN0].enable = 1;
            pHand->vpuParam[VPU_CHN0].vpuType = VPU_VOMODE_TILE192;
            pHand->vpuParam[VPU_CHN0].chnAttr.vpu_chn_size.u32Height = pconfig[i].height;
            pHand->vpuParam[VPU_CHN0].chnAttr.vpu_chn_size.u32Width = pconfig[i].width;
            pHand->vpuParam[VPU_CHN0].chnAttr.depth = 0;
            pHand->vpuParam[VPU_CHN0].fps = pconfig[i].fps;

            // venc 参数
            pHand->encParam[i].enable = pconfig[i].enable;
            pHand->encParam[i].vpuChn = VPU_CHN0;
            pHand->encParam[i].encChn = VENC_CHN_MAIN;
            pHand->encParam[i].trans_mode = pconfig[i].trans_mode;
            pHand->encParam[i].put_cb = pconfig[i].put_cb;

            // osd 参数
            pHand->osdParam[VPU_CHN0].enable = 1;
            pHand->osdParam[VPU_CHN0].left = pconfig[i].osd.left;
            pHand->osdParam[VPU_CHN0].top = pconfig[i].osd.top;
            pHand->osdParam[VPU_CHN0].fontW = pconfig[i].osd.font_w;
            pHand->osdParam[VPU_CHN0].fontH = pconfig[i].osd.font_h;
            break;
        }
        case TKL_VENC_SUB: {
            // vpu 参数
            pHand->vpuParam[VPU_CHN1].enable = 1;
            pHand->vpuParam[VPU_CHN1].vpuType = VPU_VOMODE_TILE192;
            pHand->vpuParam[VPU_CHN1].chnAttr.vpu_chn_size.u32Height = pconfig[i].height;
            pHand->vpuParam[VPU_CHN1].chnAttr.vpu_chn_size.u32Width = pconfig[i].width;
            pHand->vpuParam[VPU_CHN1].chnAttr.depth = 0;
            pHand->vpuParam[VPU_CHN1].fps = pconfig[i].fps;

            // venc 参数
            pHand->encParam[i].enable = pconfig[i].enable;
            pHand->encParam[i].vpuChn = VPU_CHN1;
            pHand->encParam[i].encChn = VENC_CHN_SUB;
            pHand->encParam[i].trans_mode = pconfig[i].trans_mode;
            pHand->encParam[i].put_cb = pconfig[i].put_cb;

            // osd 参数
            pHand->osdParam[VPU_CHN1].enable = 1;
            pHand->osdParam[VPU_CHN1].left = pconfig[i].osd.left;
            pHand->osdParam[VPU_CHN1].top = pconfig[i].osd.top;
            pHand->osdParam[VPU_CHN1].fontW = pconfig[i].osd.font_w;
            pHand->osdParam[VPU_CHN1].fontH = pconfig[i].osd.font_h;
            break;
        }

        case TKL_VENC_3RD:
        case TKL_VENC_4TH: {
            // vi参数
            //  pHand->vpuParam[i].bEnable = 0;
            // 编码参数
            pHand->encParam[i].enable = 0;
            // 第三路和第四路该平台不支持
            break;
        }

        case TKL_VENC_SNAP: {
            // jpeg venc 参数
            pHand->encParam[i].enable = pconfig[i].enable;
            pHand->encParam[i].vpuChn = VPU_CHN_JPEG;
            pHand->encParam[i].encChn = VENC_CHN_JPEG;

            if (pHand->vpuParam[VPU_CHN_JPEG].chnAttr.vpu_chn_size.u32Height != pconfig[i].height ||
                pHand->vpuParam[VPU_CHN_JPEG].chnAttr.vpu_chn_size.u32Width != pconfig[i].width) {
                pHand->encParam[i].enable = 0;
                printf("dsp.config SNAP width[%d] height[%d] need changing to width[%d] height[%d]", pconfig[i].width,
                       pconfig[i].height, pHand->vpuParam[VPU_CHN_JPEG].chnAttr.vpu_chn_size.u32Width,
                       pHand->vpuParam[VPU_CHN_JPEG].chnAttr.vpu_chn_size.u32Height);
            }
            break;
        }
        case TKL_VENC_MD: {
            pHand->encParam[i].enable = pconfig[i].enable; // 直接出YUV 不需要编码
            pHand->encParam[i].vpuChn = VPU_CHN2;
            break;
        }
        case TKL_VENC_HD: {
            // vpu 参数
            if (pconfig[i].enable) {
                pHand->vpuParam[VPU_CHN3].enable = 1;
                pHand->vpuParam[VPU_CHN3].vpuType = VPU_VOMODE_RGB888;
                pHand->vpuParam[VPU_CHN3].chnAttr.vpu_chn_size.u32Height = pconfig[i].height;
                pHand->vpuParam[VPU_CHN3].chnAttr.vpu_chn_size.u32Width = pconfig[i].width;
                pHand->vpuParam[VPU_CHN3].chnAttr.depth = 0;
                pHand->vpuParam[VPU_CHN3].fps = pconfig[i].fps;
            }

            // venc 参数
            pHand->encParam[i].enable = pconfig[i].enable; // 直接出YUV 不需要编码
            pHand->encParam[i].vpuChn = VPU_CHN3;
            break;
        }
        case TKL_VENC_QR: {
            // venc 参数
            pHand->encParam[i].enable = pconfig[i].enable; // 直接出YUV 不需要编码
            pHand->encParam[i].vpuChn = VPU_CHN2;
            pHand->vpuParam[VPU_CHN2].enable = 1;
            pHand->vpuParam[VPU_CHN2].vpuType = VPU_VOMODE_SCAN;
            pHand->vpuParam[VPU_CHN2].chnAttr.vpu_chn_size.u32Width = pconfig[i].width;
            pHand->vpuParam[VPU_CHN2].chnAttr.vpu_chn_size.u32Height = pconfig[i].height;
            pHand->vpuParam[VPU_CHN2].fps = pconfig[i].fps;
            pHand->vpuParam[VPU_CHN2].chnAttr.depth = 1;
            break;
        }
        default: {
            printf("media_type error!!!\n");
        }
        }
    }

    if (pHand->encParam[TKL_VENC_MAIN].trans_mode != pHand->encParam[TKL_VENC_SUB].trans_mode) {
        printf("err fh8636 frame trans_mode must be same\n");
        return -1;
    }

    return 0;
}

// 驱动能力配置
static void fh8636_media_driver_config(void)
{
    char vpu_cmd[48] = {0};
    int32_t channel = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    snprintf(vpu_cmd, sizeof(vpu_cmd), "vi_%d_%d", pHand->viParam.viSize.vi_size.u32Width,
             pHand->viParam.viSize.vi_size.u32Height);
    WR_PROC_DEV(VPU_PROC, vpu_cmd);

    for (channel = 0; channel < VPU_CHN_MAX; channel++) {
        if (!pHand->vpuParam[channel].enable) {
            continue;
        }

        snprintf(vpu_cmd, sizeof(vpu_cmd), "cap_%d_%d_%d", channel,
                 ALIGNTO(pHand->vpuParam[channel].chnAttr.vpu_chn_size.u32Width, 16),
                 ALIGNTO(pHand->vpuParam[channel].chnAttr.vpu_chn_size.u32Height, 16));
        WR_PROC_DEV(VPU_PROC, vpu_cmd);
    }

    return;
}

static int32_t fh8636_vi_init(void)
{
    int32_t ret = -1;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    // 设置视频输入属性(输入幅面必须和 ISP 输出幅面完全一致)
    FH_VPU_SET_GRP_INFO grp_info;
    grp_info.ycmean_en = 1;
    grp_info.ycmean_ds = 16;
    grp_info.vi_max_size.u32Width = pHand->viParam.viSize.vi_size.u32Width;
    grp_info.vi_max_size.u32Height = pHand->viParam.viSize.vi_size.u32Height;
    ret = FH_VPSS_CreateGrp(FH_GRP_ID, &grp_info);
    TY_CHECK_RETURN(ret, -1);

    ret = FH_VPSS_SetViAttr(FH_GRP_ID, &pHand->viParam.viSize);
    TY_CHECK_RETURN(ret, -1);

    // ISP 输入模式
    ret = FH_VPSS_Enable(FH_GRP_ID, VPU_MODE_ISP);
    TY_CHECK_RETURN(ret, -1);

    return 0;
}

static void fh8636_vi_deinit(void)
{
    // 关闭视频处理模块
    FH_VPSS_Disable(FH_GRP_ID);

    return;
}

static void fh8636_debug_vpu_chnattr(int32_t chan, const FH_VPU_CHN_CONFIG *chnCfg)
{
    printf("vpu chan[%d]:\n", chan);
    printf("\t vpu_chn_size[%dx%d]:\n", chnCfg->vpu_chn_size.u32Width, chnCfg->vpu_chn_size.u32Height);

    return;
}

static int32_t fh8636_vpu_init()
{
    int32_t ret = -1;
    int32_t chan = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    FH_FRAMERATE vpuframectrl = {0};
    vpuframectrl.frame_time = 1;
    FH_VPU_CHN_INFO chn_info = {0};

    // 配置vpu通道
    for (chan = 0; chan < VPU_CHN_MAX; chan++) {
        if (!pHand->vpuParam[chan].enable) {
            continue;
        }

        memset(&chn_info, 0, sizeof(FH_VPU_CHN_INFO));
        chn_info.bgm_enable = 0;
        chn_info.cpy_enable = 0;
        chn_info.sad_enable = 0;
        chn_info.bgm_ds = 0;

        chn_info.chn_max_size.u32Width = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width;
        chn_info.chn_max_size.u32Height = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height;
        chn_info.out_mode = pHand->vpuParam[chan].vpuType;
        chn_info.support_mode = 1 << pHand->vpuParam[chan].vpuType;
        chn_info.bufnum = pHand->vpuParam[chan].chnAttr.depth + 1; // 配成1出错，提示必须大于depth
        chn_info.max_stride = 0;

        ret = FH_VPSS_CreateChn(FH_GRP_ID, chan, &chn_info);
        TY_CHECK_RETURN(ret, ret);

        /*
          设置视频处理模块通道属性。修改通道属性时不需要停止视频处理模块。但建议先停止该通道的使能。
          1. 在设置通道属性后,包含帧率控制,视频冻结,绑定关系等通道配置会失效,需要重新设置
         */
        ret = FH_VPSS_SetChnAttr(FH_GRP_ID, chan, &pHand->vpuParam[chan].chnAttr);
        fh8636_debug_vpu_chnattr(chan, &pHand->vpuParam[chan].chnAttr);
        TY_CHECK_RETURN(ret, ret);

        /*
          设置视频处理模块通道输出组织格式
          1. SUB/3RD通道推荐是NV12, 方便QR/MD/HD
         */
        ret = FH_VPSS_SetVOMode(FH_GRP_ID, chan, pHand->vpuParam[chan].vpuType);
        TY_CHECK_RETURN(ret, ret);

        /*
          设置帧率控制参数
          使得vpss和venc的fps一致
        */
        printf("vpu chn: %d set fps:%d \n", chan, pHand->vpuParam[chan].fps);
        vpuframectrl.frame_count = pHand->vpuParam[chan].fps;
        ret = FH_VPSS_SetFramectrl(FH_GRP_ID, chan, &vpuframectrl);
        TY_CHECK_RETURN(ret, ret);

#if 0
        if (chan == 0) {
            int level = 0;
            ret =  FH_VPSS_SetScalerCoeff(chan, level);
            printf("FH_VPSS_SetScalerCoeff level:%d\n", level);
            TY_CHECK_RETURN(ret, ret);
        }
#endif

        // 开启通道使能
        ret = FH_VPSS_OpenChn(FH_GRP_ID, chan);
        TY_CHECK_RETURN(ret, ret);
    }

    // 配置venc通道
    int i = 0;
    for (i = 0; i <= TKL_VENC_SUB; i++) {
        if (!pHand->encParam[i].enable) {
            continue;
        }

        /*
          创建编码通道，指定通道支持的最大编码幅面，和编码格式。
          1. 需要按需要处理的最大能力创建，创建之后无法销毁。
          2. 可以指明一个通道同时支持多种不同的编码类型。
          3. 支持多种类型，分配内存时会按各类型种最大的一块内存进行内存分配，在设置通道属性时将这块内存给相应驱动使用。
         */
        ret = FH_VENC_CreateChn(FH_GRP_ID * VPU_CHN_MAX + pHand->encParam[i].encChn, &pHand->encParam[i].vencCap);
        TY_CHECK_RETURN(ret, ret);

        /*
          设置编码通道属性
          1. 在创建通道时可以指定多个通道支持MJPEG/JPEG的编码能力，但设置通道编码类型时，
          2. 同时只允许一个通道进行JPEG/MJPEG编码（JPEG和MJPEG的能力可以共存，在独立两个通道同时生效）。
         */
        printf("FH_VENC_SetChnAttr chan:%d, enc_type:%d\n", pHand->encParam[i].encChn,
               pHand->encParam[i].vencCfg.chn_attr.enc_type);
        ret = FH_VENC_SetChnAttr(FH_GRP_ID * VPU_CHN_MAX + pHand->encParam[i].encChn, &pHand->encParam[i].vencCfg);
        TY_CHECK_RETURN(ret, ret);

#if 0 // def VENC_BREATH_ON
      //  设置去呼吸效应
        FH_DEBREATH breath;
        breath.debreath_en = 1;
        breath.debreath_ratio = 16; //呼吸效应比率, 越小呼吸效应改善越明显，推荐值16 | [0-31]
        ret = FH_VENC_SetDeBreathEffect(FH_GRP_ID * VPU_CHN_MAX + i, &breath);
        TY_CHECK_RETURN(ret, ret);
#endif
        /*
          通道开启接收图像数据
          1.通道为JPEG抓图模式时，不需要调用FH_VENC_StartRecvPic/ FH_VENC_StopRecvPic。
         */
        printf("FH_VENC_StartRecvPic chan = %d, encChn = %d\n", FH_GRP_ID * VPU_CHN_MAX + pHand->encParam[i].encChn,
               pHand->encParam[i].encChn);
        ret = FH_VENC_StartRecvPic(FH_GRP_ID * VPU_CHN_MAX + pHand->encParam[i].encChn);
        TY_CHECK_RETURN(ret, ret);

        /*
        数据源到编码通道的绑定接口
         1. 一个VPU通道可以绑定到多个VENC通道。
         2. 一个VENC通道只能接收一个VPU通道的数据。
         3. 除JPEG通道外,其他通道需要满足数据源和接收者的幅面一致。
         4. 重新设置VPU/VENC通道属性会导致其相关绑定关系失效,需要重新绑定。
         5. 对JPEG通道的绑定,在完成一次抓图后会自动失效,需要再次绑定触发下一次抓图。
         */
        FH_BIND_INFO src, dst;
        src.obj_id = FH_OBJ_VPU_VO;
        src.dev_id = FH_GRP_ID;
        src.chn_id = pHand->encParam[i].vpuChn;

        dst.obj_id = FH_OBJ_ENC;
        dst.dev_id = 0;
        dst.chn_id = FH_GRP_ID * VPU_CHN_MAX + pHand->encParam[i].encChn;
        ret = FH_SYS_Bind(src, dst);
        TY_CHECK_RETURN(ret, ret);
    }

    return 0;
}

static void fh8636_vpu_deinit(void)
{
    int32_t ret = -1;
    int32_t chan = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    for (chan = 0; chan < VPU_CHN_MAX; chan++) {
        if (pHand->encParam[chan].enable) {
            continue;
        }

        /* 关闭主码流编码通道 */
        ret = FH_VENC_StopRecvPic(chan);
        if (ret != RETURN_OK) {
            printf("FH_VENC_StopRecvPic ret=0x%08X", ret);
        }
    }

    for (chan = 0; chan < VPU_CHN_MAX; chan++) {
        if (pHand->vpuParam[chan].enable) {
            continue;
        }

        /* 关闭主码流VPU通道 */
        ret = FH_VPSS_CloseChn(FH_GRP_ID, chan);
        if (ret != RETURN_OK) {
            printf("FH_VPSS_CloseChn ret=0x%08X", ret);
        }
    }

    return;
}

static int32_t fh8636_stop(void)
{
    TY_FH8636_VIDEO_HANDLE_T *hdl = __get_video_handle();
    if (!hdl->runing) {
        printf("dsp is not running!\n");
        return 0;
    }

    fh8636_vpu_deinit();
    ty_fh8636_osd_deinit();
    fh8636_vi_deinit();
    fh8636_isp_deinit();
    FH_SYS_Exit();
    hdl->runing = 0;

    return 0;
}

int __isp_bind_vi()
{
    FH_BIND_INFO src;
    FH_BIND_INFO dst;
    int ret = -1;
    src.obj_id = FH_OBJ_ISP;
    src.dev_id = 0;
    src.chn_id = 0;
    dst.obj_id = FH_OBJ_VPU_VI;
    dst.dev_id = 0;
    dst.chn_id = 0;
    ret = FH_SYS_Bind(src, dst);
    if (ret != 0) {
        printf("Error(%d - %x): FH_SYS_Bind ISP[%d] to VPU[%d]-chn[%d]\n", ret, ret, src.chn_id, dst.dev_id,
               dst.chn_id);
        return -1;
    }
    return 0;
}

static int __sys_init()
{
    int ret = -1;
    VB_CONF_S stVbConf = {0};
    FH_VB_Exit();

    fh8636_media_driver_config();

    WR_PROC_DEV(JPEG_PROC, "allchnstm_0_512000_32");
    WR_PROC_DEV(JPEG_PROC, "jpgstm_512000_32");
    WR_PROC_DEV(ENC_PROC, "stm_2048576_128");
    WR_PROC_DEV(ENC_PROC, "allchnstm_0_2048576_128");
    WR_PROC_DEV(VPU_PROC, "vofmt_0_4_2");
    WR_PROC_DEV(VPU_PROC, "vofmt_1_2_1");
    WR_PROC_DEV(VPU_PROC, "cirbuf_on");
    WR_PROC_DEV(ISP_PROC, "wdr_off");
    WR_PROC_DEV(ISP_PROC, "cir_on");
    WR_PROC_DEV(ISP_PROC, "gme_on");

    ret = FH_SYS_Init();
    TY_CHECK_RETURN(ret, ret);

    ret = FH_VB_SetConf(&stVbConf);
    if (ret) {
        printf("[FH_VB_SetConf] failed with:%x\n", ret);
        return -1;
    }

    ret = FH_VB_Init();
    if (ret) {
        printf("[FH_VB_Init] failed with:%x\n", ret);
        return -1;
    }

    return 0;
}

int32_t fh8636_start()
{
    int32_t ret = -1;
    TY_FH8636_VIDEO_HANDLE_T *hdl = __get_video_handle();

    if (hdl->runing) {
        printf("dsp has been running ...\n");
        return 0;
    }

    ret = __sys_init();
    TY_CHECK_RETURN(ret, ret);
    ret = fh8636_vi_init();
    TY_CHECK_RETURN(ret, ret);
    ret = fh8636_vpu_init();
    TY_CHECK_RETURN(ret, ret);
    ret = ty_fh8636_osd_init();
    TY_CHECK_RETURN(ret, ret);
    ret = fh8636_isp_init();
    TY_CHECK_RETURN(ret, ret);

    ret = __isp_bind_vi();
    TY_CHECK_RETURN(ret, ret);
    hdl->runing = 1;

    return 0;
}

OPERATE_RET tkl_venc_get_format(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FORMAT_T *p_format)
{
    (void)vi_chn;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    if (p_format == NULL) {
        printf("p_format is NULL\n");
        return -1;
    }
    if (venc_chn < 0 || venc_chn >= TKL_VENC_MAX) {
        printf("not support venc %d\n", venc_chn);
        return -1;
    }

    pthread_mutex_lock(&pHand->enc_mutex);
    *p_format = pHand->format[venc_chn];
    pthread_mutex_unlock(&pHand->enc_mutex);
    // printf("get_format[%d] w h:%d %d, codetype:%d\n", venc_chn, pHand->format[venc_chn].width,
    // pHand->format[venc_chn].height, p_format->codectype);
    return 0;
}

OPERATE_RET tkl_venc_set_format(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FORMAT_T *p_format)
{
    (void)vi_chn;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    if (((int)venc_chn != TKL_VENC_MAIN) && ((int)venc_chn != TKL_VENC_SUB)) {
        printf("not support venc %d\n", venc_chn);
        return -1;
    }

    if (p_format == NULL) {
        printf("p_format is NULL\n");
        return -1;
    }

    if ((p_format->codectype != TKL_CODEC_VIDEO_H264) && (p_format->codectype != TKL_CODEC_VIDEO_H265)) {
        printf("codectype %d not support\n", p_format->codectype);
        return -1;
    }

    pthread_mutex_lock(&pHand->enc_mutex);
    if (0 != memcmp(p_format, &pHand->format[venc_chn], sizeof(TKL_VENC_FORMAT_T))) {
        memcpy(&pHand->format[venc_chn], p_format, sizeof(TKL_VENC_FORMAT_T));
        pHand->format_flag[venc_chn] = 1;
    } else {
        printf("set_format is same, not update\n");
    }
    pthread_mutex_unlock(&pHand->enc_mutex);

    printf("set_format[%d] codetype fps gop bitrate:%d %d %d, profile:%d\n", venc_chn, p_format->h264_attr.fps,
           p_format->h264_attr.gop, p_format->h264_attr.bitrate, p_format->h264_attr.profile);
    printf("set_format[%d] w h:%d %d, codetype:%d\n", venc_chn, pHand->format[venc_chn].width,
           pHand->format[venc_chn].height, p_format->codectype);

    return 0;
}

static int32_t fh8636_set_format(TKL_VENC_CHN_E venc_chn)
{
    int32_t ret = -1;
    int32_t chan = venc_chn;
    int32_t venc = venc_chn;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    TKL_VENC_FORMAT_T *p_format = NULL;
    FH_VENC_CHN_CONFIG venc_cfg = {0};

    if (((int)venc_chn != TKL_VENC_MAIN) && ((int)venc_chn != TKL_VENC_SUB)) {
        printf("not support venc %d\n", venc_chn);
        return -1;
    }

    pthread_mutex_lock(&pHand->enc_mutex);
    p_format = &pHand->format[venc_chn];
    pHand->encParam[venc_chn].width = p_format->width;
    pHand->encParam[venc_chn].height = p_format->height;
    pHand->vpuParam[venc_chn].chnAttr.vpu_chn_size.u32Width = pHand->encParam[venc_chn].width;
    pHand->vpuParam[venc_chn].chnAttr.vpu_chn_size.u32Height = pHand->encParam[venc_chn].height;
    if (p_format->codectype == TKL_CODEC_VIDEO_H264) {
        switch (p_format->h264_attr.profile) {
        case TKL_VENC_H264_PROFILE_MAIN:
            pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.profile = H264_PROFILE_MAIN;
            break;

        case TKL_VENC_H264_PROFILE_HIGH:
            pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.profile = H264_PROFILE_HIGH;
            break;

        case TKL_VENC_H264_PROFILE_BASELINE:
        default:
            pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.profile = H264_PROFILE_BASELINE;
            break;
        }

        pHand->encParam[venc_chn].vencCfg.rc_attr.h264_vbr.bitrate = p_format->h264_attr.bitrate * 1000;
        pHand->encParam[venc_chn].vencCfg.rc_attr.h264_vbr.FrameRate.frame_count = (FH_UINT16)p_format->h264_attr.fps;
        pHand->encParam[venc_chn].vencCfg.rc_attr.h264_vbr.FrameRate.frame_time = 1;
        pHand->encParam[venc_chn].vencCfg.chn_attr.enc_type = FH_NORMAL_H264;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.i_frame_intterval = p_format->h264_attr.gop;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.size.u32Width = p_format->width;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h264_attr.size.u32Height = p_format->height;
        venc_cfg = pHand->encParam[venc_chn].vencCfg;
    } else if (p_format->codectype == TKL_CODEC_VIDEO_H265) {
        pHand->encParam[venc_chn].vencCfg.chn_attr.enc_type = FH_NORMAL_H265;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h265_attr.profile = H265_PROFILE_MAIN;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h265_attr.i_frame_intterval = p_format->h265_attr.gop;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h265_attr.size.u32Width = p_format->width;
        pHand->encParam[venc_chn].vencCfg.chn_attr.h265_attr.size.u32Height = p_format->height;
        if ((p_format->h265_attr.bitrate <= 1000) && (venc_chn == (int)TKL_VENC_MAIN)) { /// 针对低码率优化编码参数
            venc_cfg = pHand->encParam[venc_chn].vencCfg;
            venc_cfg.rc_attr.rc_type = FH_RC_H265_AVBR;
            venc_cfg.rc_attr.h265_avbr.bitrate = p_format->h265_attr.bitrate * 1000;
            venc_cfg.rc_attr.h265_avbr.FrameRate.frame_count = (FH_UINT16)p_format->h265_attr.fps;
            venc_cfg.rc_attr.h265_avbr.FrameRate.frame_time = 1;
            venc_cfg.rc_attr.h265_avbr.init_qp = 38;
            venc_cfg.rc_attr.h265_avbr.ImaxQP = 40;
            venc_cfg.rc_attr.h265_avbr.IminQP = 26;
            venc_cfg.rc_attr.h265_avbr.PmaxQP = 48;
            venc_cfg.rc_attr.h265_avbr.PminQP = 40;
            venc_cfg.rc_attr.h265_avbr.maxrate_percent = 200;
            venc_cfg.rc_attr.h265_avbr.IFrmMaxBits = 0;
            venc_cfg.rc_attr.h265_avbr.IP_QPDelta = 3;
            venc_cfg.rc_attr.h265_avbr.I_BitProp = H265_BREATH_I_BITPROP;
            venc_cfg.rc_attr.h265_avbr.P_BitProp = H265_BREATH_P_BITPROP;
            venc_cfg.rc_attr.h265_avbr.fluctuate_level = 0;
            venc_cfg.rc_attr.h265_avbr.stillrate_percent = 25;
            venc_cfg.rc_attr.h265_avbr.maxstillqp = 34;
        } else {
            pHand->encParam[venc_chn].vencCfg.rc_attr.h265_vbr.bitrate = p_format->h265_attr.bitrate * 1000 * 7 / 8;
            pHand->encParam[venc_chn].vencCfg.rc_attr.h265_vbr.FrameRate.frame_count =
                (FH_UINT16)p_format->h265_attr.fps;
            pHand->encParam[venc_chn].vencCfg.rc_attr.h265_vbr.FrameRate.frame_time = 1;
            venc_cfg = pHand->encParam[venc_chn].vencCfg;
        }
    } else {
        pthread_mutex_unlock(&pHand->enc_mutex);
        printf("not support venc %d codectype:%d\n", venc_chn, p_format->codectype);
        return -1;
    }
    pHand->format_flag[venc_chn] = 0;
    pthread_mutex_unlock(&pHand->enc_mutex);

    /* 关闭主码流VPU通道 */
    // 配置vpu通道
    if (pHand->vpuParam[chan].enable) {
        ret = FH_VPSS_CloseChn(FH_GRP_ID, chan);
        if (ret != RETURN_OK) {
            printf("Error(%d - %x): FH_VPSS_CloseChn\n", ret, ret);
            return -1;
        }
    }

    if (pHand->encParam[venc].enable) {
        /* 关闭主码流编码通道 */
        printf("set_format stop:%d %d\n", venc, pHand->encParam[venc].encChn);
        ret = FH_VENC_StopRecvPic(pHand->encParam[venc].encChn);
        if (ret != RETURN_OK) {
            printf("Error(%d - %x): FH_VENC_StopRecvPic\n", ret, ret);
            return -1;
        }
    }

    // 配置vpu通道
    if (pHand->vpuParam[chan].enable) {
        /*
            设置视频处理模块通道属性。修改通道属性时不需要停止视频处理模块。但建议先停止该通道的使能。
            1. 在设置通道属性后,包含帧率控制,视频冻结,绑定关系等通道配置会失效,需要重新设置
            */
        ret = FH_VPSS_SetChnAttr(FH_GRP_ID, chan, &pHand->vpuParam[chan].chnAttr);
        fh8636_debug_vpu_chnattr(chan, &pHand->vpuParam[chan].chnAttr);
        TY_CHECK_RETURN(ret, ret);

        /*
            设置视频处理模块通道输出组织格式
            1. SUB/3RD通道推荐是NV12, 方便QR/MD/HD
            */
        ret = FH_VPSS_SetVOMode(FH_GRP_ID, chan, pHand->vpuParam[chan].vpuType);
        TY_CHECK_RETURN(ret, ret);

        /*
            设置帧率控制参数
            使得vpss和venc的fps一致
        */
        if (pHand->encParam[chan].enable) {
            if (FH_RC_H264_VBR == pHand->encParam[chan].vencCfg.rc_attr.rc_type) {
                ret = FH_VPSS_SetFramectrl(FH_GRP_ID, chan, &pHand->encParam[chan].vencCfg.rc_attr.h264_vbr.FrameRate);
                TY_CHECK_RETURN(ret, ret);
            } else if (FH_RC_H265_VBR == pHand->encParam[chan].vencCfg.rc_attr.rc_type) {
                ret = FH_VPSS_SetFramectrl(FH_GRP_ID, chan, &pHand->encParam[chan].vencCfg.rc_attr.h265_vbr.FrameRate);
                TY_CHECK_RETURN(ret, ret);
            }
        }

        // 调整缩放属性解决子码流锯齿问题
        if (chan <= VPU_CHN1) {
            if (chan == VPU_CHN0) {
                ret = FH_VPSS_SetScalerCoeff(FH_GRP_ID, chan, 15);
            } else {
                ret = FH_VPSS_SetScalerCoeff(FH_GRP_ID, chan, 8);
            }
            TY_CHECK_RETURN(ret, ret);
        }

        // 开启通道使能
        ret = FH_VPSS_OpenChn(FH_GRP_ID, chan);
        TY_CHECK_RETURN(ret, ret);
    }

    // 配置venc通道
    if (pHand->encParam[venc].enable) {
        /*
          设置编码通道属性
          1. 在创建通道时可以指定多个通道支持MJPEG/JPEG的编码能力，但设置通道编码类型时，
          2. 同时只允许一个通道进行JPEG/MJPEG编码（JPEG和MJPEG的能力可以共存，在独立两个通道同时生效）。
         */
        ret = FH_VENC_SetChnAttr(pHand->encParam[venc].encChn, &venc_cfg);
        TY_CHECK_RETURN(ret, ret);

        /*
          通道开启接收图像数据
          1.通道为JPEG抓图模式时，不需要调用FH_VENC_StartRecvPic/ FH_VENC_StopRecvPic。
         */
        printf("chan = %d, encChn = %d\n", pHand->encParam[venc].encChn, pHand->encParam[venc].encChn);
        ret = FH_VENC_StartRecvPic(pHand->encParam[venc].encChn);
        TY_CHECK_RETURN(ret, ret);

        /*
        数据源到编码通道的绑定接口
         1. 一个VPU通道可以绑定到多个VENC通道。
         2. 一个VENC通道只能接收一个VPU通道的数据。
         3. 除JPEG通道外,其他通道需要满足数据源和接收者的幅面一致。
         4. 重新设置VPU/VENC通道属性会导致其相关绑定关系失效,需要重新绑定。
         5. 对JPEG通道的绑定,在完成一次抓图后会自动失效,需要再次绑定触发下一次抓图。
         */
        FH_BIND_INFO src, dst;
        src.obj_id = FH_OBJ_VPU_VO;
        src.dev_id = FH_GRP_ID;
        src.chn_id = pHand->encParam[chan].vpuChn;

        dst.obj_id = FH_OBJ_ENC;
        dst.dev_id = 0;
        dst.chn_id = pHand->encParam[chan].encChn;

        FH_SYS_UnBindbyDst(dst);
        ret = FH_SYS_Bind(src, dst);
        TY_CHECK_RETURN(ret, ret);
    }
    return 0;
}

static void *fh8636_venc_proc(void *arg)
{
    int32_t ret;
    uint32_t chan = 0;
    uint32_t i;
    uint32_t size = 0;
    FH_VENC_STREAM stream;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    TKL_VENC_FRAME_T frame = {0};
    (void)arg;

    prctl(PR_SET_NAME, THRD_DSP_VIDEO);
    while (pHand->thread_status) {
        /*阻塞模式下,获取一帧H264或者H265数据*/
        ret = FH_VENC_GetStream_Block(FH_STREAM_ALL & (~(FH_JPEG)), &stream);
        if (ret != RETURN_OK) {
            printf("FH_VENC_GetStream_Block failed, ret = %08x\n", ret);
            usleep(500 * 1000);
            continue;
        }

        size = 0;
        if (FH_STREAM_H264 == stream.stmtype) {
            chan = stream.chan;
            frame.frametype = stream.h264_stream.frame_type == FH_FRAME_I ? TKL_VIDEO_I_FRAME : TKL_VIDEO_PB_FRAME;
            frame.height = pHand->encParam[chan].height;
            frame.width = pHand->encParam[chan].width;
            frame.pts = stream.h264_stream.time_stamp;
            frame.timestamp = stream.h264_stream.time_stamp;
            frame.codectype = TKL_CODEC_VIDEO_H264;

            for (i = 0; i < stream.h264_stream.nalu_cnt; i++) {
                size += stream.h264_stream.nalu[i].length;
            }
            if (size + SEI_FRAME_BUFFER_SIZE > MAX_FRAME_BUFFER_SIZE) {
                printf("h264 frame out of range size:%u\n", size);
                FH_VENC_ReleaseStream(&stream);
                continue;
            }
            size = 0;

            for (i = 0; i < stream.h264_stream.nalu_cnt; i++) {
                memcpy(pHand->frame_buf + size + SEI_FRAME_BUFFER_SIZE, stream.h264_stream.nalu[i].start,
                       stream.h264_stream.nalu[i].length);
                size += stream.h264_stream.nalu[i].length;
            }
        } else if (FH_STREAM_H265 == stream.stmtype) {
            chan = stream.chan;
            frame.frametype = stream.h265_stream.frame_type == FH_FRAME_I ? TKL_VIDEO_I_FRAME : TKL_VIDEO_PB_FRAME;
            frame.height = pHand->encParam[chan].height;
            frame.width = pHand->encParam[chan].width;
            frame.pts = stream.h265_stream.time_stamp;
            frame.timestamp = stream.h265_stream.time_stamp;
            frame.codectype = TKL_CODEC_VIDEO_H265;
            for (i = 0; i < stream.h265_stream.nalu_cnt; i++) {
                size += stream.h265_stream.nalu[i].length;
            }
            if (size + SEI_FRAME_BUFFER_SIZE > MAX_FRAME_BUFFER_SIZE) {
                printf("h265 frame out of range size:%u\n", size);
                FH_VENC_ReleaseStream(&stream);
                continue;
            }
            size = 0;

            for (i = 0; i < stream.h265_stream.nalu_cnt; i++) {
                memcpy(pHand->frame_buf + size + SEI_FRAME_BUFFER_SIZE, stream.h265_stream.nalu[i].start,
                       stream.h265_stream.nalu[i].length);
                size += stream.h265_stream.nalu[i].length;
            }
        } else {
            printf("stream.stmtype = %d, not sppourt!!!\n", stream.stmtype);
            FH_VENC_ReleaseStream(&stream);
            continue;
        }

        frame.pbuf = pHand->frame_buf + SEI_FRAME_BUFFER_SIZE;
        frame.used_size = size;
        frame.buf_size = MAX_FRAME_BUFFER_SIZE - SEI_FRAME_BUFFER_SIZE;
        if (0 != pHand->encParam[chan].put_cb(&frame)) {
            printf("put frame failed!\n");
        }

        /*必须和FH_VENC_GetStream配套调用,以释放码流资源*/
        FH_VENC_ReleaseStream(&stream);

        if (pHand->format_flag[TKL_VENC_MAIN] == 1) {
            fh8636_set_format(TKL_VENC_MAIN);
        }
        if (pHand->format_flag[TKL_VENC_SUB] == 1) {
            fh8636_set_format(TKL_VENC_SUB);
        }
    }

    return NULL;
}

static void __scale_down2(char *src_data, char *dst_data, int dst_width)
{
    int x = 0;
    for (x = 0; x < dst_width - 1; x += 2) {
        dst_data[0] = src_data[0];
        dst_data[1] = src_data[2];
        dst_data += 2;
        src_data += 4;
    }
}

/*
 * @fn int __linear_scale_down2(char *in_data, TY_VIDEO_SCALE_PATA_T paras, char *out_data)
 * 线性缩放函数
 * @retval 0 成功
 * @retval 非0 失败
 * @attention 仅支持Y缩小，而且宽高恰好都是缩小1/2
 */
static int __linear_scale_down2(char *in_data, TY_VIDEO_SCALE_PATA_T paras, char *out_data)
{
    int y = 0;
    char *pSrc = NULL;
    char *pDst = NULL;
    // 注意这里宽度最好是正真的stride，如果in_data指针有边缘则直接取宽度做stride是有问题的
    int dst_stride = paras.dstWidth;
    int src_stride = paras.srcWidth;

    pSrc = in_data;
    pDst = out_data;

    for (y = 0; y < paras.dstHeight; y++) {
        __scale_down2(pSrc, pDst, paras.dstWidth);
        pSrc += (src_stride << 1);
        pDst += dst_stride;
    }
    return 0;
}

// 1/4 缩放  仅支持Y缩小，而且宽高恰好都是缩小1/4
static void __scale_down4(char *src_data, char *dst_data, int dst_width)
{
    int x = 0;
    for (x = 0; x < dst_width - 1; x += 2) {
        dst_data[0] = src_data[0];
        dst_data[1] = src_data[4];
        dst_data += 2;
        src_data += 8;
    }
}

static int __linear_scale_down4(char *in_data, TY_VIDEO_SCALE_PATA_T paras, char *out_data)
{
    int y = 0;
    char *pSrc = NULL;
    char *pDst = NULL;
    int dst_stride = paras.dstWidth;
    int src_stride = paras.srcWidth;

    // 640*360 ->160*90
    pSrc = in_data;
    pDst = out_data;

    for (y = 0; y < paras.dstHeight; y++) {
        __scale_down4(pSrc, pDst, paras.dstWidth);
        pSrc += (src_stride << 2);
        pDst += dst_stride;
    }
    return 0;
}

/**
 * @fn ty_yuv_zoom_proce(char * yuvData, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *pframe)
 * yuv 数据的缩放处理
 * @param[in] yuvData yuv 数据
 * @param[in] pFrame YUV数据信息传出
 * @retval 0 成功 非0成功
 * @attention 无。
 */
int ty_yuv_zoom_proce(char *yuvData, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *pframe)
{
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    TY_VIDEO_SCALE_PATA_T scalePara = {0};
    int len = 0;
    int ret = 0;
    int32_t chan = FH_APP_YUV_CHANNEL;

    if (pHand->encParam[venc_chn].width == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
        pHand->encParam[venc_chn].height == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        pframe->used_size = pHand->encParam[venc_chn].width * pHand->encParam[venc_chn].height * 3 / 2;
        memcpy(pframe->pbuf, yuvData, pframe->used_size);
        pframe->height = pHand->encParam[venc_chn].height;
        pframe->width = pHand->encParam[venc_chn].width;
    } else if (pHand->encParam[venc_chn].width * 4 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
               pHand->encParam[venc_chn].height * 4 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        // 1/4 缩放
        scalePara.srcWidth = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width;
        scalePara.srcHeight = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height;
        scalePara.dstHeight = pHand->encParam[venc_chn].height;
        scalePara.dstWidth = pHand->encParam[venc_chn].width;
        len = scalePara.dstHeight * scalePara.dstWidth * 3 / 2;
        memset(pframe->pbuf, 0, len);
        ret = __linear_scale_down4(yuvData, scalePara, pframe->pbuf);
        if (ret != 0) {
            printf("__linear_scale_down4 err\n");
            return -1;
        }
        pframe->used_size = len;
        pframe->height = scalePara.dstHeight;
        pframe->width = scalePara.dstWidth;
    } else if (pHand->encParam[venc_chn].width * 2 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
               pHand->encParam[venc_chn].height * 2 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        // 1/2 缩放
        scalePara.srcWidth = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width;
        scalePara.srcHeight = pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height;
        scalePara.dstHeight = pHand->encParam[venc_chn].height;
        scalePara.dstWidth = pHand->encParam[venc_chn].width;
        len = scalePara.dstHeight * scalePara.dstWidth * 3 / 2;
        memset(pframe->pbuf, 0, len);
        ret = __linear_scale_down2(yuvData, scalePara, pframe->pbuf);
        if (ret != 0) {
            printf("__linear_scale_down2 err\n");
            return -1;
        }
        pframe->used_size = len;
        pframe->height = scalePara.dstHeight;
        pframe->width = scalePara.dstWidth;
    } else {
        printf("get yuv:%d failed, need w h:%d %d, but w h:%d %d\n", venc_chn, pHand->encParam[venc_chn].width,
               pHand->encParam[venc_chn].height, pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width,
               pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height);
        return -1;
    }
    return 0;
}

// md的yuv，时间区域叠加黑块
static int32_t __md_osd_black(char *p_y)
{
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int w = pHand->encParam[TKL_VENC_MD].width;
    int multiple = 1;
    int font_width = 0;
    int font_left = 0;
    int font_height = 0;
    int font_top = 0;
    int chan = FH_APP_YUV_CHANNEL; // md 用chn 1
    int j = 0;
    int i = 0;

    if (chan > VPU_CHN1) {
        // md 通道使用非主子码流时，不叠加黑块
        return 0;
    }

    if (pHand->encParam[TKL_VENC_MD].width == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
        pHand->encParam[TKL_VENC_MD].height == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        multiple = 1; // 未缩放
    } else if (pHand->encParam[TKL_VENC_MD].width * 4 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
               pHand->encParam[TKL_VENC_MD].height * 4 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        // 1/4 缩放
        multiple = 4;
    } else if (pHand->encParam[TKL_VENC_MD].width * 2 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width &&
               pHand->encParam[TKL_VENC_MD].height * 2 == pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height) {
        // 1/2 缩放
        multiple = 2;
    } else {
        printf("not support, md w h:%d %d, yuv w h:%d %d\n", pHand->encParam[TKL_VENC_MD].width,
               pHand->encParam[TKL_VENC_MD].height, pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width,
               pHand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height);
        return -1;
    }

    font_width = 20 * pHand->osdParam[VPU_CHN1].fontW / multiple / 2; //  20*24/2/2 =120  multiple = 2
    font_left = pHand->osdParam[VPU_CHN1].left / multiple;            // 32/2 = 16
    font_height = pHand->osdParam[VPU_CHN1].fontH / multiple;         // 24/2 = 12
    font_top = pHand->osdParam[VPU_CHN1].top / multiple;              // 18/2 = 9
    for (j = font_top; j < font_height + font_top; j++) {
        for (i = font_left; i < font_left + font_width; i++) {
            p_y[j * w + i] = 0;
        }
    }

    return 0;
}
/* 获取VPU YUV数据 */
static int32_t fh8636_capture_yuv(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *frame)
{
    (void)vi_chn;
    FH_VPU_STREAM_ADV yuv_data = {0};
    int32_t ret = -1;
    int32_t chan = FH_APP_YUV_CHANNEL;
    uint32_t w = 0;
    uint32_t h = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    if ((int)venc_chn != TKL_VENC_MD && (int)venc_chn != TKL_VENC_QR) {
        printf("not support chn:%d\n", venc_chn);
        return -1;
    }

    /* 获取VPU通道参数 */
    if (!pHand->vpuParam[chan].enable) {
        printf("VPU CHN %d not enabled!\n", chan);
        return -1;
    }

    w = pHand->encParam[venc_chn].width; // 目标宽高
    h = pHand->encParam[venc_chn].height;

    /* 分配yuv格式转换源数据空间 */
    if (w * h * 3 / 2 > frame->buf_size) {
        printf("ch:%d w = %d, h = %d, frame buf_size[%d] for yuv is not enough!\n", venc_chn, w, h, frame->buf_size);
        return -1;
    }

    /* 获取一帧YUV数据 */
    pthread_mutex_lock(&pHand->yuv_mutex);
    ret = FH_VPSS_GetChnFrameAdv(FH_GRP_ID, chan, &yuv_data, 1000);
    pthread_mutex_unlock(&pHand->yuv_mutex);
    TY_CHECK_RETURN(ret, ret);

    if (VPU_VOMODE_SCAN != pHand->vpuParam[chan].vpuType) {
        printf("chan[%d] yuv type is %d, tile format cannot be analyzed\n", chan, pHand->vpuParam[chan].vpuType);
        return -1;
    }

    ret = ty_yuv_zoom_proce((char *)yuv_data.frm_scan.luma.data.vbase, venc_chn, frame);
    TY_CHECK_RETURN(ret, -1);

    // md的yuv上osd 影响md判断，os区域叠加黑块
    if ((int)venc_chn == TKL_VENC_MD) {
        ret = __md_osd_black(frame->pbuf);
        TY_CHECK_RETURN(ret, -1);
    }

    return 0;
}

static int32_t fh8636_capture_stream(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *frame)
{
    (void)vi_chn;
    int ret = -1;
    FH_VENC_STREAM stream = {0};
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    FH_STREAM_TYPE stream_type = -1;
    uint32_t len = 0;
    uint32_t i = 0;

    if (pHand->encParam[venc_chn].vencCfg.chn_attr.enc_type == FH_NORMAL_H264) {
        stream_type = FH_STREAM_H264;
    } else if (pHand->encParam[venc_chn].vencCfg.chn_attr.enc_type == FH_NORMAL_H265) {
        stream_type = FH_STREAM_H265;
    } else {
        printf("venc chn:%d, codectype:%d, not support\n", venc_chn,
               pHand->encParam[venc_chn].vencCfg.chn_attr.enc_type);
        return -1;
    }

    if (frame->pbuf == NULL) {
        printf("venc chn:%d, buff is NULL\n", venc_chn);
        return -1;
    }

    /*阻塞模式下,获取一帧H264或者H265数据*/
    ret = FH_VENC_GetStream_Block(stream_type, &stream);
    if (ret != RETURN_OK) {
        printf("FH_VENC_GetStream_Block failed, ret = %08x\n", ret);
        return -1;
    }

    if (FH_STREAM_H264 == stream.stmtype) {
        for (i = 0; i < stream.h264_stream.nalu_cnt; i++) {
            len += stream.h264_stream.nalu[i].length;
        }
        if (len > frame->buf_size) {
            FH_VENC_ReleaseStream(&stream);
            printf("frame buf_size[%d] is not enough, need %d\n", frame->buf_size, len);
            return -1;
        }

        frame->frametype = stream.h264_stream.frame_type == FH_FRAME_I ? TKL_VIDEO_I_FRAME : TKL_VIDEO_PB_FRAME;
        frame->height = pHand->encParam[venc_chn].height;
        frame->width = pHand->encParam[venc_chn].width;
        frame->pts = stream.h264_stream.time_stamp;
        frame->timestamp = stream.h264_stream.time_stamp;
        frame->used_size = 0;
        frame->codectype = TKL_CODEC_VIDEO_H264;

        for (i = 0; i < stream.h264_stream.nalu_cnt; i++) {
            memcpy(frame->pbuf + frame->used_size, stream.h264_stream.nalu[i].start, stream.h264_stream.nalu[i].length);
            frame->used_size += stream.h264_stream.nalu[i].length;
        }
        FH_VENC_ReleaseStream(&stream);
    } else if (FH_STREAM_H265 == stream.stmtype) {
        for (i = 0; i < stream.h265_stream.nalu_cnt; i++) {
            len += stream.h265_stream.nalu[i].length;
        }
        if (len > frame->buf_size) {
            FH_VENC_ReleaseStream(&stream);
            printf("frame buf_size[%d] is not enough, need %d\n", frame->buf_size, len);
            return -1;
        }

        frame->frametype = stream.h265_stream.frame_type == FH_FRAME_I ? TKL_VIDEO_I_FRAME : TKL_VIDEO_PB_FRAME;
        frame->height = pHand->encParam[venc_chn].height;
        frame->width = pHand->encParam[venc_chn].width;
        frame->pts = stream.h265_stream.time_stamp;
        frame->timestamp = stream.h265_stream.time_stamp;
        frame->used_size = 0;
        frame->codectype = TKL_CODEC_VIDEO_H265;

        for (i = 0; i < stream.h265_stream.nalu_cnt; i++) {
            memcpy(frame->pbuf + frame->used_size, stream.h265_stream.nalu[i].start, stream.h265_stream.nalu[i].length);
            frame->used_size += stream.h265_stream.nalu[i].length;
        }
        FH_VENC_ReleaseStream(&stream);
    } else {
        printf("stream.stmtype = %d, not sppourt!!!\n", stream.stmtype);
        return -1;
    }
    return 0;
}

/* jpeg抓图 */
static int32_t fh8636_capture_jpeg(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *frame)
{
    (void)vi_chn;
    int ret = -1;
    FH_VENC_STREAM stream = {0};
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int32_t vpu_channel = pHand->encParam[venc_chn].vpuChn;
    int32_t jpeg_channel = pHand->encParam[venc_chn].encChn;

    if (!pHand->jpeg_channel_inited) {
        /* 第一次调用需要创建jpeg编码通道 */
        if (!pHand->vpuParam[vpu_channel].enable) {
            printf("Error: we use channel %d for JPEG, Please enable channel %d firstly!\n", VPU_CHN_JPEG,
                   VPU_CHN_JPEG);
            return -1;
        }

        ret = FH_VENC_CreateChn(jpeg_channel, &pHand->encParam[venc_chn].vencCap);
        TY_CHECK_RETURN(ret, ret);

        ret = FH_VENC_SetChnAttr(jpeg_channel, &pHand->encParam[venc_chn].vencCfg);
        TY_CHECK_RETURN(ret, ret);

        pHand->jpeg_channel_inited = 1;
    }

    /* jpeg编码通道需要每次都绑定 */
    FH_BIND_INFO src, dst;
    src.obj_id = FH_OBJ_VPU_VO;
    src.dev_id = FH_GRP_ID;
    src.chn_id = vpu_channel;

    dst.obj_id = FH_OBJ_JPEG;
    dst.dev_id = 0;
    dst.chn_id = jpeg_channel;
    ret = FH_SYS_Bind(src, dst);
    TY_CHECK_RETURN(ret, ret);

    /* 获取jpeg数据 */
    ret = FH_VENC_GetStream_Block(FH_STREAM_JPEG, &stream);
    TY_CHECK_RETURN(ret, ret);

    // stream.jpeg_stream.chan: jpeg channel
    // stream.jpeg_stream.start: jpeg start mem addr
    // stream.jpeg_stream.length: jpeg mem length
    frame->frametype = TKL_VIDEO_I_FRAME;
    frame->height = pHand->encParam[venc_chn].width;
    frame->width = pHand->encParam[venc_chn].height;
    frame->pts =
        stream.jpeg_stream.time_stamp; // web拉流需要pts，单位为 us 该平台获取的 pts为0，手动添加 // fixed ms or us
    frame->timestamp = stream.jpeg_stream.time_stamp;
    if (stream.jpeg_stream.length <= frame->buf_size) {
        memcpy(frame->pbuf, stream.jpeg_stream.start, stream.jpeg_stream.length);
        frame->used_size = stream.jpeg_stream.length;
    } else {
        ret = -1;
        printf("JPEG frame buf_size[%d] is not enough, need %d\n", frame->buf_size, stream.jpeg_stream.length);
    }

    FH_VENC_ReleaseStream(&stream);
    printf("Capture JPEG OK, size=%d\n\n", stream.jpeg_stream.length);

    ret = FH_SYS_UnBindbyDst(dst);
    TY_CHECK_RETURN(ret, ret);

    return ret;
}

/**
 * @fn int ty_dev_venc_stop(int vi_chn, E_DSP_MEDIA_VIDEO_TYPE type)
 * 停止编码
 * @param[in] vi_chn 预留多目
 * @param[in] type 停止编码的流类型
 * @retval 0 成功 非0成功
 * @attention 无。
 */
int ty_dev_venc_stop(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn)
{
    if (vi_chn != 0) {
        printf("ty_dev_venc_stop wrong vi_chn = %d\n", vi_chn);
        return -1;
    }

    if (TKL_VENC_0 != venc_chn && TKL_VENC_1 != venc_chn) {
        printf("ty_dev_venc_stop venc_chn = %d, not support!\n", venc_chn);
        return 0;
    }

    return 0;
}

/***********************************************************
*  函数名: tkl_venc_get_frame
*  功能：获取视频码流，包括主子码流，动检人形YUV数据等
*  参数: Input，vi_chn，多目预留
        Input，venc_chn，编码通道
        Input，Output,pframe->pbuf和pframe->buf_size 入参，
              返回时，pbuf由底层cp数据，used_size 为实际数据长度
*
*  返回值: <0 失败，=0 成功
***********************************************************/
OPERATE_RET tkl_venc_get_frame(TKL_VI_CHN_E vi_chn, TKL_VENC_CHN_E venc_chn, TKL_VENC_FRAME_T *pframe)
{
    int ret = -1;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    if (vi_chn != 0) {
        printf("ty_dev_venc_get wrong vi_chn = %d\n", vi_chn);
        return -1;
    }

    if (pHand->isInit != 1) {
        printf("venc[%d] is not init\n", venc_chn);
        return -1;
    }

    if (venc_chn < TKL_VENC_4) {
        if (pHand->encParam[venc_chn].trans_mode == TKL_VENC_PUSH_FRAME_BY_CALLBACK) { // 回调模式不支持获取
            printf("put frame by cb, not support get\n");
            return -1;
        }
        ret = fh8636_capture_stream(vi_chn, venc_chn, pframe);
    } else if (TKL_VENC_4 == venc_chn) {
        pthread_mutex_lock(&pHand->enc_mutex);
        ret = fh8636_capture_jpeg(vi_chn, venc_chn, pframe);
        pthread_mutex_unlock(&pHand->enc_mutex);
    } else if (venc_chn < TKL_VENC_MAX) {
        ret = fh8636_capture_yuv(vi_chn, venc_chn, pframe);
    } else {
        printf("tkl_venc_get_frame wrong venc_chn = %d, \n", venc_chn);
        return -1;
    }

    return ret;
}

int ty_dev_venc_deinit(int vi_chn)
{
    int32_t ret = -1;
    if (vi_chn != 0) {
        printf("ty_dev_venc_deinit wrong vi_chn = %d\n", vi_chn);
        return -1;
    }
    ret = fh8636_stop();
    TY_CHECK_RETURN(ret, -1);

    return 0;
}

OPERATE_RET tkl_venc_set_time_cb(TKL_VENC_TIME_CB cb)
{
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    pHand->cb = cb;
    printf("tkl_venc_set_time_cb end\n");
    return 0;
}
